@@ -20,14 +20,16 @@
 import React from 'react';
 import cockpit from 'cockpit';
 
+import { ExclamationCircleIcon } from "@patternfly/react-icons";
+
 import $ from "jquery";
 import { docker } from './docker';
-import { atomic } from './atomic';
 import { util } from "./util";
 import { search } from "./search";
 
 import * as Listing from 'cockpit-components-listing.jsx';
 import * as Select from 'cockpit-components-select.jsx';
+import { EmptyStatePanel } from "cockpit-components-empty-state.jsx";
 import moment from 'moment';
 
 const _ = cockpit.gettext;
@@ -40,6 +42,8 @@ class Dropdown extends React.Component {
     }
 
     handleClick(event) {
+        event.preventDefault();
+
         if (event.button !== 0)
             return;
 
@@ -51,18 +55,18 @@ class Dropdown extends React.Component {
     render() {
         return (
             <div className="btn-group">
-                <button className="btn btn-default" type="button" data-value="0" onClick={this.handleClick}>
+                <button className="pf-c-button pf-m-secondary" type="button" data-value="0" onClick={this.handleClick}>
                     <span>{ this.props.actions[0].label }</span>
                 </button>
                 <button className="btn btn-default dropdown-toggle" data-toggle="dropdown">
-                    <div className="caret" />
+                    <i className="fa fa-caret-down pf-c-context-selector__toggle-icon" aria-hidden="true" />
                 </button>
                 <ul className="dropdown-menu dropdown-menu-right" role="menu">
                     {
                         this.props.actions.map(function (action, index) {
                             return (
                                 <li key={index} className={ action.disabled ? 'disabled' : '' }>
-                                    <a data-value={index} role="link" tabIndex="0" onClick={this.handleClick}>{action.label}</a>
+                                    <a data-value={index} href="#" onClick={this.handleClick}>{action.label}</a>
                                 </li>
                             );
                         }.bind(this))
@@ -144,6 +148,7 @@ class ContainerProblems extends React.Component {
     }
 
     onItemClick(event) {
+        event.preventDefault();
         cockpit.jump(event.currentTarget.dataset.url, cockpit.transport.host);
     }
 
@@ -151,7 +156,7 @@ class ContainerProblems extends React.Component {
         var problem = this.props.problem;
         var problem_cursors = [];
         for (var i = 0; i < problem.length; i++) {
-            problem_cursors.push(<a key={i} data-url={problem[i][0]} className='list-group-item' role="link" tabIndex="0" onClick={this.onItemClick}>
+            problem_cursors.push(<a key={i} data-url={problem[i][0]} className='list-group-item' href="#" onClick={this.onItemClick}>
                 <span className="pficon pficon-warning-triangle-o fa-lg" />
                 {problem[i][1]}
             </a>);
@@ -319,9 +324,11 @@ export class ContainerList extends React.Component {
 
             var actions = (
                 <>
-                    <button className="btn btn-danger btn-delete pficon pficon-delete"
-                            onClick={ this.deleteContainer.bind(this, container) } />
-                    <button className="btn btn-default"
+                    <button className="pf-c-button pf-m-danger btn-delete"
+                            onClick={ this.deleteContainer.bind(this, container) }>
+                        <span className="pficon pficon-delete" />
+                    </button>
+                    <button className="pf-c-button pf-m-secondary"
                             disabled={isRunning}
                             data-container-id={container.Id}
                             data-toggle="modal" data-target="#container-commit-dialog">
@@ -418,96 +425,12 @@ class ImageDetails extends React.Component {
     }
 }
 
-class ImageSecurity extends React.Component {
-    render() {
-        var info = this.props.info;
-        var text, rows;
-        var args = {
-            time: info.finishedTime.format('MMM Do'),
-            type: info.scanType,
-            count: info.vulnerabilities.length
-        };
-
-        if (info.successful === false) {
-            text = _("The scan from $time ($type) was not successful.");
-        } else if (info.vulnerabilities.length === 0) {
-            text = _("The scan from $time ($type) found no vulnerabilities.");
-        } else {
-            text = cockpit.ngettext('The scan from $time ($type) found one vulnerability:',
-                                    'The scan from $time ($type) found $count vulnerabilities:',
-                                    info.vulnerabilities.length);
-
-            rows = info.vulnerabilities.map(function (vulnerability) {
-                return (
-                    <div key={vulnerability.description}
-                         className="vulnerability-row-ct-docker" title={vulnerability.description}>
-                        <span>{vulnerability.title}</span>
-                        <span className="pull-right">{vulnerability.severity}</span>
-                    </div>
-                );
-            });
-        }
-
-        return (
-            <div>
-                <div className="listing-ct-body-header">
-                    { cockpit.format(text, args) }
-                </div>
-                <div>
-                    {rows}
-                </div>
-            </div>
-        );
-    }
-}
-
 export class ImageInline extends React.Component {
-    constructor(props) {
-        super(props);
-        this.state = {
-            vulnerableInfos: {}
-        };
-        this.vulnerableInfoChanged = this.vulnerableInfoChanged.bind(this);
-    }
-
-    vulnerableInfoChanged(event, infos) {
-        this.setState({ vulnerableInfos: infos });
-    }
-
-    componentDidMount() {
-        atomic.addEventListener('vulnerableInfoChanged', this.vulnerableInfoChanged);
-    }
-
-    componentWillUnmount() {
-        atomic.removeEventListener('vulnerableInfoChanged', this.vulnerableInfoChanged);
-    }
-
     render() {
         var image = this.props.image;
 
-        if (!image) {
-            return (
-                <div className="curtains-ct blank-slate-pf">
-                    <div className="blank-slate-pf-icon">
-                        <i className="fa fa-exclamation-circle" />
-                    </div>
-                    <h1>{_("This image does not exist.")}</h1>
-                </div>
-            );
-        }
-
-        var vulnerableInfo = this.state.vulnerableInfos[image.Id.replace(/^sha256:/, '')];
-
-        if (vulnerableInfo) {
-            return (
-                <div className="listing-ct-inline">
-                    <h3>{_("Details")}</h3>
-                    <ImageDetails image={image} />
-                    <h3>{_("Security")}</h3>
-                    <ImageSecurity image={image} info={vulnerableInfo} />
-                </div>
-            );
-        }
+        if (!image)
+            return <EmptyStatePanel icon={ExclamationCircleIcon} title={ _("This image does not exist.") } />;
 
         return (
             <div className="listing-ct-inline">
@@ -524,12 +447,10 @@ export class ImageList extends React.Component {
         this.state = {
             images: [],
             pulling: [],
-            vulnerableInfos: {}
         };
         this.handleSearchImageClick = this.handleSearchImageClick.bind(this);
         this.imagesChanged = this.imagesChanged.bind(this);
         this.pullingChanged = this.pullingChanged.bind(this);
-        this.vulnerableInfoChanged = this.vulnerableInfoChanged.bind(this);
         this.renderRow = this.renderRow.bind(this);
     }
 
@@ -580,54 +501,28 @@ export class ImageList extends React.Component {
         this.setState({ pulling: this.props.client.pulling });
     }
 
-    vulnerableInfoChanged(event, infos) {
-        this.setState({ vulnerableInfos: infos });
-    }
-
     componentDidMount() {
         $(this.props.client).on('image.containers', this.imagesChanged);
         $(this.props.client).on('pulling.containers', this.pullingChanged);
-
-        atomic.addEventListener('vulnerableInfoChanged', this.vulnerableInfoChanged);
     }
 
     componentWillUnmount() {
         $(this.props.client).off('image.containers', this.imagesChanged);
         $(this.props.client).off('pulling.containers', this.pullingChanged);
-
-        atomic.removeEventListener('vulnerableInfoChanged', this.vulnerableInfoChanged);
     }
 
     renderRow(image) {
-        var vulnerabilityColumn = '';
-
-        var vulnerableInfo = this.state.vulnerableInfos[image.Id.replace(/^sha256:/, '')];
-        var count;
-
-        if (vulnerableInfo) {
-            count = vulnerableInfo.vulnerabilities.length;
-            if (count > 0)
-                vulnerabilityColumn = (
-                    <div>
-                        <span className="pficon pficon-warning-triangle-o" />
-                        &nbsp;
-                        { cockpit.format(cockpit.ngettext('1 Vulnerability', '$0 Vulnerabilities', count), count) }
-                    </div>
-                );
-        }
-
         var element;
         if (this.props.client.waiting[image.Id]) {
             element = <div className="spinner" />;
         } else {
-            element = <button className="btn btn-default btn-control-ct fa fa-play"
+            element = <button className="pf-c-button pf-m-secondary play-button btn-sm"
                 onClick={ this.showRunImageDialog }
-                data-image={image.Id} />;
+                data-image={image.Id}><span className="fa fa-play" /></button>;
         }
 
         var columns = [
             { name: image.RepoTags[0], header: true },
-            vulnerabilityColumn,
             moment.unix(image.Created).calendar(),
             cockpit.format_bytes(image.VirtualSize),
             {
@@ -644,20 +539,11 @@ export class ImageList extends React.Component {
             data: { image: image }
         });
 
-        if (vulnerableInfo !== undefined) {
-            tabs.push({
-                name: _("Security"),
-                renderer: ImageSecurity,
-                data: {
-                    image: image,
-                    info: vulnerableInfo,
-                }
-            });
-        }
-
         var actions = (
-            <button className="btn btn-danger btn-delete pficon pficon-delete"
-                    onClick={ this.deleteImage.bind(this, image) } />
+            <button className="pf-c-button pf-m-danger btn-delete"
+                    onClick={ this.deleteImage.bind(this, image) }>
+                <span className="pficon pficon-delete" />
+            </button>
         );
 
         return <Listing.ListingRow key={image.Id}
@@ -676,9 +562,13 @@ export class ImageList extends React.Component {
 
         var imageRows = filtered.map(this.renderRow, this);
 
-        var getNewImageAction = <a role="link" tabIndex="0" onClick={this.handleSearchImageClick} className="card-pf-link-with-icon pull-right">
-            <span className="pficon pficon-add-circle-o" />{_("Get new image")}
-        </a>;
+        var getNewImageAction = <div className="pull-right">
+            <button onClick={this.handleSearchImageClick} className="link-button">
+                <span className="pficon pficon-add-circle-o" />
+                {" "}
+                {_("Get new image")}
+            </button>
+        </div>;
 
         var columnTitles = [_("Name"), '', _("Created"), _("Size"), ''];
 
@@ -697,7 +587,7 @@ export class ImageList extends React.Component {
                 );
             }
 
-            return <p className="status">{job.name}: {job.status} {detail}</p>;
+            return <p key={job.registry + job.name + job.tag} className="status">{job.registry}/{job.name}:{job.tag} - {job.status} {detail}</p>;
         });
 
         var emptyCaption;
