@@ -21,12 +21,12 @@ import cockpit from "cockpit";
 
 import React from "react";
 import { OnOffSwitch } from "cockpit-components-onoff.jsx";
-import { OverlayTrigger, Tooltip } from "patternfly-react";
+import { Button, Tooltip, TooltipPosition } from "@patternfly/react-core";
 
 import * as Select from "cockpit-components-select.jsx";
 import { show_modal_dialog } from "cockpit-components-dialog.jsx";
 
-import "form-layout.less";
+import "form-layout.scss";
 
 const _ = cockpit.gettext;
 
@@ -327,6 +327,7 @@ export class KdumpPage extends React.Component {
         // only consider primary mouse button
         if (!e || e.button !== 0)
             return;
+        e.preventDefault();
         var self = this;
         var settings = { };
         Object.keys(self.props.kdumpStatus.config).forEach((key) => {
@@ -403,7 +404,7 @@ export class KdumpPage extends React.Component {
         // this.storeLocation(this.props.kdumpStatus.config);
         var settingsLink;
         if (targetCanChange)
-            settingsLink = <a href="#" tabIndex="0" onClick={this.handleSettingsClick}>{ kdumpLocation }</a>;
+            settingsLink = <span><button className="link-button" onClick={this.handleSettingsClick}>{ kdumpLocation }</button></span>;
         else
             settingsLink = <span>{ kdumpLocation }</span>;
         var reservedMemory;
@@ -451,20 +452,20 @@ export class KdumpPage extends React.Component {
             if (this.props.reservedMemory == 0) {
                 const tooltip = _("No memory reserved. Append a crashkernel option to the kernel command line (e.g. in /etc/default/grub) to reserve memory at boot time. Example: crashkernel=512M");
                 serviceHint = (
-                    <OverlayTrigger overlay={ <Tooltip id="tip-service">{tooltip}</Tooltip> } placement="bottom">
+                    <Tooltip id="tip-service" content={tooltip} position={TooltipPosition.bottom}>
                         <span className="popover-ct-kdump fa fa-lg fa-info-circle" />
-                    </OverlayTrigger>
+                    </Tooltip>
                 );
             }
-            kdumpServiceDetails = <a className="popover-ct-kdump" href="#" tabIndex="0" onClick={this.handleServiceDetailsClick}>{serviceDescription}{serviceHint}</a>;
+            kdumpServiceDetails = <button role="link" className="popover-ct-kdump link-button" onClick={this.handleServiceDetailsClick}>{serviceDescription}{serviceHint}</button>;
         } else if (this.props.kdumpStatus && !this.props.kdumpStatus.installed) {
             const tooltip = _("Kdump service not installed. Please ensure package kexec-tools is installed.");
             kdumpServiceDetails = (
-                <OverlayTrigger overlay={ <Tooltip id="tip-service">{tooltip}</Tooltip> } placement="bottom">
-                    <a tabIndex="0" className="popover-ct-kdump">
+                <Tooltip id="tip-service" content={tooltip} position={TooltipPosition.bottom}>
+                    <button className="popover-ct-kdump link-button">
                         <span className="fa fa-lg fa-info-circle" />
-                    </a>
-                </OverlayTrigger>
+                    </button>
+                </Tooltip>
             );
         }
         var serviceWaiting;
@@ -474,18 +475,18 @@ export class KdumpPage extends React.Component {
         var testButton;
         if (serviceRunning) {
             testButton = (
-                <button className="btn btn-default" onClick={this.handleTestSettingsClick}>
+                <Button variant="secondary" onClick={this.handleTestSettingsClick}>
                     {_("Test Configuration")}
-                </button>
+                </Button>
             );
         } else {
             const tooltip = _("Test is only available while the kdump service is running.");
             testButton = (
-                <OverlayTrigger overlay={ <Tooltip id="tip-test">{tooltip}</Tooltip> } placement="top">
-                    <button className="btn btn-default disabled">
+                <Tooltip id="tip-test" content={tooltip}>
+                    <Button variant="secondary" isDisabled>
                         {_("Test Configuration")}
-                    </button>
-                </OverlayTrigger>
+                    </Button>
+                </Tooltip>
             );
         }
         const tooltip_info = _("This will test the kdump configuration by crashing the kernel.");
@@ -508,11 +509,11 @@ export class KdumpPage extends React.Component {
 
                     <div role="group">
                         {testButton}
-                        <a tabIndex="0" className="popover-ct-kdump">
-                            <OverlayTrigger overlay={ <Tooltip id="tip-test-info">{tooltip_info}</Tooltip> } placement="top">
+                        <button className="popover-ct-kdump link-button">
+                            <Tooltip id="tip-test-info" content={tooltip_info}>
                                 <span className="fa fa-lg fa-info-circle" />
-                            </OverlayTrigger>
-                        </a>
+                            </Tooltip>
+                        </button>
                     </div>
                 </form>
             </div>
