@@ -19,8 +19,8 @@
 
 import React from 'react';
 import PropTypes from 'prop-types';
-import { FormGroup, HelpBlock, Modal, OverlayTrigger, Tooltip, TypeAheadSelect } from 'patternfly-react';
-import { Button } from '@patternfly/react-core';
+import { FormGroup, HelpBlock, Modal, TypeAheadSelect } from 'patternfly-react';
+import { Button, Tooltip, TooltipPosition } from '@patternfly/react-core';
 
 import cockpit from 'cockpit';
 import { MachinesConnectionSelector } from '../machinesConnectionSelector.jsx';
@@ -183,7 +183,7 @@ function validateParams(vmParams) {
             );
         }
     }
-    if (vmParams.unattendedInstallation && !vmParams.rootPassword && !vmParams.userPassword)
+    if (vmParams.unattendedInstallation && !vmParams.rootPassword && (permission.user.name == 'root' || !vmParams.userPassword))
         validationFailed.password = _("Please set a root or a user password");
 
     return validationFailed;
@@ -213,7 +213,7 @@ const NameRow = ({ vmName, onValueChanged, validationFailed }) => {
     );
 };
 
-const SourceRow = ({ connectionName, source, sourceType, networks, nodeDevices, providerName, os, osInfoList, downloadOSSupported, onValueChanged, validationFailed }) => {
+const SourceRow = ({ connectionName, source, sourceType, networks, nodeDevices, os, osInfoList, downloadOSSupported, onValueChanged, validationFailed }) => {
     let installationSource;
     let installationSourceId;
     let installationSourceWarning;
@@ -295,12 +295,11 @@ const SourceRow = ({ connectionName, source, sourceType, networks, nodeDevices,
                     <Select.SelectEntry data={LOCAL_INSTALL_MEDIA_SOURCE}
                         key={LOCAL_INSTALL_MEDIA_SOURCE}>{_("Local Install Media")}</Select.SelectEntry>
                     <Select.SelectEntry data={URL_SOURCE} key={URL_SOURCE}>{_("URL")}</Select.SelectEntry>
-                    { providerName == 'LibvirtDBus' &&
                     <Select.SelectEntry title={connectionName == 'session' ? _("Network Boot is available only when using System connection") : null}
                         disabled={connectionName == 'session'}
                         data={PXE_SOURCE}
                         key={PXE_SOURCE}>{_("Network Boot (PXE)")}
-                    </Select.SelectEntry>}
+                    </Select.SelectEntry>
                 </Select.Select>
             </>}
 
@@ -405,9 +404,9 @@ const UnattendedRow = ({ validationFailed, unattendedDisabled, unattendedInstall
     );
     if (unattendedDisabled) {
         unattendedInstallationCheckbox = (
-            <OverlayTrigger overlay={ <Tooltip id='os-unattended-installation-tooltip'>{ _("The selected Operating System does not support unattended installation") }</Tooltip> } placement='left'>
+            <Tooltip id='os-unattended-installation-tooltip' content={_("The selected Operating System does not support unattended installation")} position={TooltipPosition.left}>
                 {unattendedInstallationCheckbox}
-            </OverlayTrigger>
+            </Tooltip>
         );
     }
 
@@ -446,7 +445,7 @@ const UnattendedRow = ({ validationFailed, unattendedDisabled, unattendedInstall
                         {profile == 'desktop' && <p className="text-info">{_("Leave the password blank if you do not wish to have a root account created")}</p>}
                     </HelpBlock>
                 </FormGroup>
-                {profile == 'desktop' && <>
+                {permission.user.name != 'root' && profile == 'desktop' && <>
                     <label htmlFor='user-password' className='control-label'>
                         {cockpit.format(_("Password for $0"), permission.user.name)}
                     </label>
@@ -762,7 +761,7 @@ class CreateVmModal extends React.Component {
     }
 
     onCreateClicked() {
-        const { dispatch, providerName, storagePools, close, onAddErrorNotification, osInfoList, vms } = this.props;
+        const { dispatch, storagePools, close, onAddErrorNotification, osInfoList, vms } = this.props;
 
         const validation = validateParams({ ...this.state, osInfoList, vms: vms.filter(vm => vm.connectionName == this.state.connectionName) });
         if (Object.getOwnPropertyNames(validation).length > 0) {
@@ -796,7 +795,7 @@ class CreateVmModal extends React.Component {
                 () => {
                     close();
 
-                    if (providerName == 'LibvirtDBus' && this.state.storagePool === "NewVolume") {
+                    if (this.state.storagePool === "NewVolume") {
                         const storagePool = storagePools.find(pool => pool.connectionName === this.state.connectionName && pool.name === "default");
                         if (storagePool)
                             storagePoolRefresh(storagePool.connectionName, storagePool.id);
@@ -813,7 +812,7 @@ class CreateVmModal extends React.Component {
     }
 
     render() {
-        const { nodeMaxMemory, nodeDevices, networks, osInfoList, loggedUser, providerName, storagePools, vms } = this.props;
+        const { nodeMaxMemory, nodeDevices, networks, osInfoList, loggedUser, storagePools, vms } = this.props;
         const validationFailed = this.state.validate && validateParams({ ...this.state, osInfoList, vms: vms.filter(vm => vm.connectionName == this.state.connectionName) });
         let startVmCheckbox = (
             <label className="checkbox-inline">
@@ -826,9 +825,11 @@ class CreateVmModal extends React.Component {
         );
         if (this.state.unattendedInstallation) {
             startVmCheckbox = (
-                <OverlayTrigger overlay={ <Tooltip id='virt-install-not-available-tooltip'>{ _("Setting the user passwords for unattended installation requires starting the VM when creating it") }</Tooltip> } placement='left'>
+                <Tooltip id='virt-install-not-available-tooltip'
+                         position={TooltipPosition.left}
+                         content={_("Setting the user passwords for unattended installation requires starting the VM when creating it")}>
                     {startVmCheckbox}
-                </OverlayTrigger>
+                </Tooltip>
             );
         }
 
@@ -862,7 +863,6 @@ class CreateVmModal extends React.Component {
                     connectionName={this.state.connectionName}
                     networks={networks.filter(network => network.connectionName == this.state.connectionName)}
                     nodeDevices={nodeDevices.filter(nodeDevice => nodeDevice.connectionName == this.state.connectionName)}
-                    providerName={providerName}
                     source={this.state.source}
                     sourceType={this.state.sourceType}
                     os={this.state.os}
@@ -940,15 +940,15 @@ class CreateVmModal extends React.Component {
                     {dialogBody}
                 </Modal.Body>
                 <Modal.Footer>
-                    {this.state.inProgress && <div className="spinner spinner-sm pull-left" />}
-                    <Button variant='secondary' className='btn-cancel' onClick={ this.props.close }>
-                        {_("Cancel")}
-                    </Button>
                     <Button variant="primary"
                             isDisabled={Object.getOwnPropertyNames(validationFailed).length > 0}
                             onClick={this.onCreateClicked}>
                         {this.props.mode == 'create' ? _("Create") : _("Import")}
                     </Button>
+                    <Button variant='link' className='btn-cancel' onClick={ this.props.close }>
+                        {_("Cancel")}
+                    </Button>
+                    {this.state.inProgress && <div className="spinner spinner-sm pull-right" />}
                 </Modal.Footer>
             </Modal>
         );
@@ -969,14 +969,14 @@ export class CreateVmAction extends React.Component {
     }
 
     componentDidMount() {
-        cockpit.spawn(['which', 'virt-install'], { err: 'message' })
+        cockpit.spawn(['which', 'virt-install'], { err: 'ignore' })
                 .then(() => {
                     this.setState({ virtInstallAvailable: true });
-                    cockpit.spawn(['virt-install', '--install=?'])
+                    cockpit.spawn(['virt-install', '--install=?'], { err: 'ignore' })
                             .then(() => this.setState({ downloadOSSupported: true }),
                                   () => this.setState({ downloadOSSupported: false }));
 
-                    cockpit.spawn(['virt-install', '--unattended=?'])
+                    cockpit.spawn(['virt-install', '--unattended=?'], { err: 'ignore' })
                             .then(() => this.setState({ unattendedSupported: true }),
                                   () => this.setState({ unattendedSupported: false }));
                 },
@@ -1005,31 +1005,25 @@ export class CreateVmAction extends React.Component {
             testdata = "disabledOsInfo";
         else if (!this.state.virtInstallAvailable)
             testdata = "disabledVirtInstall";
-        else if (this.state.downloadOSSupported === undefined)
-            testdata = "disabledDownloadOS";
+        else if (this.state.downloadOSSupported === undefined || this.state.unattendedSupported === undefined)
+            testdata = "disabledCheckingFeatures";
         let createButton = (
-            <Button isDisabled={!(this.props.systemInfo.osInfoList &&
-                               (this.state.virtInstallAvailable !== undefined ||
-                                   (this.state.virtInstallAvailable &&
-                                       (this.state.downloadOSSupported === undefined || this.state.unattendedSupported === undefined))))}
+            <Button isDisabled={testdata !== undefined}
                     testdata={testdata}
                     id={this.props.mode == 'create' ? 'create-new-vm' : 'import-vm-disk'}
                     variant='secondary'
-                    style={!this.state.virtInstallAvailable ? { pointerEvents: 'none' } : null} // Fixes OverlayTrigger not showing up
                     onClick={this.open}>
                 {this.props.mode == 'create' ? _("Create VM") : _("Import VM")}
             </Button>
         );
         if (!this.state.virtInstallAvailable)
             createButton = (
-                <OverlayTrigger overlay={
-                    <Tooltip id='virt-install-not-available-tooltip'>
-                        {_("virt-install package needs to be installed on the system in order to create new VMs")}
-                    </Tooltip>} placement='top'>
+                <Tooltip id='virt-install-not-available-tooltip'
+                         content={_("virt-install package needs to be installed on the system in order to create new VMs")}>
                     <span>
                         {createButton}
                     </span>
-                </OverlayTrigger>
+                </Tooltip>
             );
 
         return (
@@ -1038,7 +1032,6 @@ export class CreateVmAction extends React.Component {
                 { this.state.showModal &&
                 <CreateVmModal
                     mode={this.props.mode}
-                    providerName={this.props.providerName}
                     close={this.close} dispatch={this.props.dispatch}
                     networks={this.props.networks}
                     nodeDevices={this.props.nodeDevices}
@@ -1064,7 +1057,6 @@ CreateVmAction.propTypes = {
     nodeDevices: PropTypes.array.isRequired,
     nodeMaxMemory: PropTypes.number,
     onAddErrorNotification: PropTypes.func.isRequired,
-    providerName: PropTypes.string.isRequired,
     systemInfo: PropTypes.object.isRequired,
 };
 CreateVmAction.defaultProps = {
