@@ -24,6 +24,10 @@ const _ = cockpit.gettext;
 export const LIBVIRT_SESSION_CONNECTION = 'session';
 export const LIBVIRT_SYSTEM_CONNECTION = 'system';
 
+export function dummyVmsFilter(vms, uiVms) {
+    return uiVms.filter(uiVm => vms.find(vm => vm.name == uiVm.name && vm.connectionName == uiVm.connectionName) === undefined);
+}
+
 export function toReadableNumber(number) {
     if (number < 1) {
         return number.toFixed(2);
@@ -131,6 +135,13 @@ export function isEmpty(str) {
     return (!str || str.length === 0);
 }
 
+export function isObjectEmpty(obj) {
+    if (!obj)
+        return false;
+
+    return Object.keys(obj).length === 0;
+}
+
 export function arrayEquals(arr1, arr2) {
     if (arr1.length !== arr2.length) {
         return false;
@@ -174,9 +185,6 @@ export function getTodayYearShifted(yearDifference) {
 
 const transform = {
     autostart: {
-        disable: _("disabled"),
-        enable: _("enabled"),
-        /* The following keys are used in LibvirtDBus provider */
         false: _("disabled"),
         true: _("enabled"),
     },
@@ -601,6 +609,27 @@ function getVmDisksMap(vms, connectionName) {
     return vmDisksMap;
 }
 
+/**
+ * Returns a string which represent disk target of volume in VM using the said volume.
+ *
+ * @param {object} vm
+ * @param {object} storagePool
+ * @param {string} volumeName
+ * @returns {string}
+ */
+export function getStorageVolumeDiskTarget(vm, storagePool, volumeName) {
+    const disks = vm.disks || [];
+    const targetPath = storagePool.target ? storagePool.target.path : '';
+    const volumePath = targetPath + '/' + volumeName;
+
+    for (const i in disks) {
+        const disk = disks[i];
+        if ((disk.type == 'volume' && disk.source.volume == volumeName && disk.source.pool == storagePool.name) ||
+            (disk.type == 'file' && disk.source.file == volumePath))
+            return disk.target;
+    }
+}
+
 /**
  * Returns a object of key-value pairs of Storage Volume names mapping
  * to arrays of VM names using the relevant Storage Volume
@@ -676,3 +705,91 @@ export function getDefaultVolumeFormat(pool) {
 
     return undefined;
 }
+
+/**
+ * Returns whetever disk property of VM's inactive XML has changed
+ * compared to live XML.
+ * Mainly used for readonly and shareable properties.
+ *
+ * @param {object} vm
+ * @param {string} diskTarget
+ * @param {string} property
+ * @returns {boolean}
+ */
+export function diskPropertyChanged(vm, diskTarget, property) {
+    const disk = vm.disks[diskTarget];
+    const inactiveDisk = vm.inactiveXML.disks[diskTarget];
+
+    if (disk && inactiveDisk) // only persistent disks
+        return disk[property] !== inactiveDisk[property];
+    else
+        return false;
+}
+
+/**
+ * Returns an identifying value which can be used as disk name.
+ * Can be file path, url, pool/volume or disk device type (fallback)
+ *
+ * @param {object} disk
+ * @returns {string}
+ */
+export function getDiskFullName(disk) {
+    let name;
+
+    if (["file", "block", "dir"].includes(disk.type)) {
+        // file path
+        let path;
+        if (disk.type === "file")
+            path = disk.source.file;
+        else if (disk.type === "block")
+            path = disk.source.dev;
+        else if (disk.type === "dir")
+            path = disk.source.dir;
+
+        name = path;
+    } else if (disk.type === "network") {
+        // url
+        name = disk.source.name;
+    } else if (disk.type === "volume") {
+        // pool/volume
+        name = disk.source.pool + '/' + disk.source.volume;
+    }
+
+    // fallback
+    if (name === undefined)
+        name = disk.device;
+
+    return name;
+}
+
+/**
+ * Returns a shortened pretty version of disk name.
+ * File path or pool/volume gets parsed, rest is unmodified.
+ *
+ * @param {object} disk
+ * @returns {string}
+ */
+export function getDiskPrettyName(disk) {
+    let name = getDiskFullName(disk);
+
+    if (["file", "block", "dir"].includes(disk.type) || disk.type === "volume") {
+        const parts = name.split('/');
+        name = parts[parts.length - 1];
+    }
+
+    return name;
+}
+
+export function getNextAvailableTarget(existingTargets, busType) {
+    let i = 0;
+    let prefix = 'vd';
+    if (busType !== 'virtio')
+        prefix = 'sd';
+
+    while (i < 26) {
+        const target = prefix + `${String.fromCharCode(97 + i)}`;
+        if (!existingTargets.includes(target))
+            return target;
+        i++;
+    }
+}
