@@ -17,16 +17,15 @@
  * along with Cockpit; If not, see <http://www.gnu.org/licenses/>.
  */
 
+import '../../src/base1/patternfly-cockpit.scss';
 import cockpit from "cockpit";
 import React from "react";
 import ReactDOM from "react-dom";
 import {
     ListView,
     Modal,
-    OverlayTrigger,
-    Tooltip
 } from "patternfly-react";
-import { Alert, Button } from '@patternfly/react-core';
+import { Alert, Button, Tooltip } from '@patternfly/react-core';
 import { ExclamationCircleIcon, TrashIcon } from '@patternfly/react-icons';
 
 import firewall from "./firewall-client.js";
@@ -35,6 +34,8 @@ import { OnOffSwitch } from "cockpit-components-onoff.jsx";
 import { ModalError } from "cockpit-components-inline-notification.jsx";
 import { EmptyStatePanel } from "cockpit-components-empty-state.jsx";
 
+import { superuser } from "superuser.jsx";
+
 import "page.scss";
 import "table.css";
 import "form-layout.scss";
@@ -42,6 +43,8 @@ import "./networking.css";
 
 const _ = cockpit.gettext;
 
+superuser.reload_page_on_change();
+
 function ServiceRow(props) {
     var tcp = props.service.ports.filter(p => p.protocol.toUpperCase() == 'TCP');
     var udp = props.service.ports.filter(p => p.protocol.toUpperCase() == 'UDP');
@@ -61,22 +64,10 @@ function ServiceRow(props) {
         event.stopPropagation();
     }
 
-    var deleteButton;
-    if (props.readonly) {
-        deleteButton = (
-            <OverlayTrigger className="pull-right" placement="top"
-                            overlay={ <Tooltip id="tip-auth">{ _("You are not authorized to modify the firewall.") }</Tooltip> }>
-                <span>
-                    <Button key={props.service.id + "-delete-button"} variant="danger" aria-label={cockpit.format(_("Not authorized to remove service $0"), props.service.id)} style={{ pointerEvents: 'none' }} isDisabled><TrashIcon /></Button>
-                </span>
-            </OverlayTrigger>
-        );
-    } else {
-        deleteButton = <Button key={props.service.id + "-delete-button"} variant="danger" onClick={onRemoveService} aria-label={cockpit.format(_("Remove service $0"), props.service.id)}><TrashIcon /></Button>;
-    }
+    var deleteButton = <Button key={props.service.id + "-delete-button"} variant="danger" onClick={onRemoveService} aria-label={cockpit.format(_("Remove service $0"), props.service.id)}><TrashIcon /></Button>;
 
     var columns = [
-        { name: props.service.name, header: true },
+        { name: props.service.id, header: true },
         <div key={props.service.id + "tcp"}>
             { tcp.map(p => p.port).join(', ') }
         </div>,
@@ -96,7 +87,7 @@ function ServiceRow(props) {
             <ul>{props.service.includes.map(s => {
                 const service = firewall.services[s];
                 if (service && service.description)
-                    return <li key={service.id}><strong>{service.name}</strong>: {service.description}</li>;
+                    return <li key={service.id}><strong>{service.id}</strong>: {service.description}</li>;
             })} </ul></>;
     }
     const simpleBody = <>{description}{includes}</>;
@@ -105,7 +96,25 @@ function ServiceRow(props) {
                        rowId={props.service.id}
                        columns={columns}
                        simpleBody={simpleBody}
-                       listingActions={[deleteButton]} />;
+                       listingActions={!props.readonly && deleteButton} />;
+}
+
+function PortRow(props) {
+    const columns = [
+        <i key={props.zone.id + "-additional-ports"}>{ _("Additional ports") }</i>,
+        props.zone.ports
+                .filter(p => p.protocol === "tcp")
+                .map(p => p.port)
+                .join(", "),
+        props.zone.ports
+                .filter(p => p.protocol === "udp")
+                .map(p => p.port)
+                .join(", "),
+        null
+    ];
+    return <ListingRow key={props.zone.id + "-ports"}
+                       rowId={props.zone.id + "-ports"}
+                       columns={columns} />;
 }
 
 function ZoneSection(props) {
@@ -120,30 +129,21 @@ function ZoneSection(props) {
     let deleteButton;
     if (props.readonly) {
         deleteButton = (
-            <OverlayTrigger placement="top"
-                            overlay={ <Tooltip id="tip-auth">{ _("You are not authorized to modify the firewall.") }</Tooltip> }>
-                <Button variant="danger" aria-label={cockpit.format(_("Not authorized to remove zone $0"), props.zone.id)} isDisabled><span className="pficon pficon-delete" /></Button>
-            </OverlayTrigger>
+            <Tooltip id="tip-auth" content={ _("You are not authorized to modify the firewall.") }>
+                <span>
+                    <Button variant="danger" aria-label={cockpit.format(_("Not authorized to remove zone $0"), props.zone.id)} isDisabled><span className="pficon pficon-delete" /></Button>
+                </span>
+            </Tooltip>
         );
     } else {
         deleteButton = <Button variant="danger" onClick={onRemoveZone} aria-label={cockpit.format(_("Remove zone $0"), props.zone.id)}><span className="pficon pficon-delete" /></Button>;
     }
 
-    let addServiceAction;
-    if (firewall.readonly) {
-        addServiceAction = (
-            <OverlayTrigger placement="top"
-                                overlay={ <Tooltip id="tip-auth">{ _("You are not authorized to modify the firewall.") }</Tooltip> }>
-                <Button variant="primary" className="pull-right add-services-button" aria-label={cockpit.format(_("Not authorized to add services to zone $0"), props.zone.id)} isDisabled> {_("Add Services")} </Button>
-            </OverlayTrigger>
-        );
-    } else {
-        addServiceAction = (
-            <Button variant="primary" onClick={() => props.openServicesDialog(props.zone.id, props.zone.id)} className="add-services-button" aria-label={cockpit.format(_("Add services to zone $0"), props.zone.id)}>
-                {_("Add Services")}
-            </Button>
-        );
-    }
+    const addServiceAction = (
+        <Button variant="primary" onClick={() => props.openServicesDialog(props.zone.id, props.zone.id)} className="add-services-button" aria-label={cockpit.format(_("Add services to zone $0"), props.zone.id)}>
+            {_("Add Services")}
+        </Button>
+    );
 
     return <div className="zone-section" data-id={props.zone.id}>
         <div className="zone-section-heading">
@@ -154,7 +154,7 @@ function ZoneSection(props) {
                     { props.zone.source.length > 0 && <span className="zone-section-target"><strong>{_("Addresses")}</strong> {props.zone.source.join(", ")}</span> }
                 </div>
             </span>
-            <div className="zone-section-buttons">{deleteButton}{addServiceAction}</div>
+            { !firewall.readonly && <div className="zone-section-buttons">{deleteButton}{addServiceAction}</div> }
         </div>
         {props.zone.services.length > 0 &&
         <Listing columnTitles={[_("Service"), _("TCP"), _("UDP"), ""]}
@@ -167,6 +167,11 @@ function ZoneSection(props) {
                                        readonly={firewall.readonly} />;
             })
             }
+            { props.zone.ports.length > 0 &&
+            <PortRow key={props.zone.id + "-ports"}
+                     zone={props.zone}
+                     readonly={firewall.readonly} />}
+
         </Listing>
         }
     </div>;
@@ -234,12 +239,11 @@ class AddServicesModal extends React.Component {
             selected: new Set(),
             filter: "",
             custom: false,
-            generate_name: true,
+            generate_custom_id: true,
             tcp_error: "",
             udp_error: "",
             avail_services: null,
             custom_id: "",
-            custom_name: "",
             custom_tcp_ports: [],
             custom_udp_ports: [],
             custom_tcp_value: "",
@@ -250,12 +254,11 @@ class AddServicesModal extends React.Component {
         this.save = this.save.bind(this);
         this.onFilterChanged = this.onFilterChanged.bind(this);
         this.onToggleService = this.onToggleService.bind(this);
-        this.setName = this.setName.bind(this);
+        this.setId = this.setId.bind(this);
         this.getName = this.getName.bind(this);
         this.validate = this.validate.bind(this);
         this.createPorts = this.createPorts.bind(this);
         this.parseServices = this.parseServices.bind(this);
-        this.generateName = this.generateName.bind(this);
         this.onToggleType = this.onToggleType.bind(this);
     }
 
@@ -269,7 +272,7 @@ class AddServicesModal extends React.Component {
     save() {
         let p;
         if (this.state.custom) {
-            p = firewall.createService(this.state.custom_id, this.state.custom_name, this.createPorts(), this.props.zoneId);
+            p = firewall.createService(this.state.custom_id, this.createPorts(), this.props.zoneId);
         } else {
             p = firewall.addServices(this.props.zoneId, [...this.state.selected]);
         }
@@ -336,10 +339,10 @@ class AddServicesModal extends React.Component {
         return ret;
     }
 
-    setName(event) {
+    setId(event) {
         this.setState({
-            custom_name: event.target.value,
-            generate_name: false,
+            custom_id: event.target.value,
+            generate_custom_id: event.target.value.length === 0,
         });
     }
 
@@ -351,23 +354,6 @@ class AddServicesModal extends React.Component {
             return port;
     }
 
-    generateName(all_ports) {
-        var name = "";
-        if (all_ports.length === 1) {
-            const known = this.state.avail_services[all_ports[0]];
-            if (known)
-                name = known.description || known.name;
-            else
-                name = all_ports[0];
-        } else if (all_ports.length > 1) {
-            name = all_ports.slice(0, 3).map(this.getName)
-                    .join(", ");
-            if (all_ports.length > 3)
-                name += "...";
-        }
-        return name;
-    }
-
     getPortNumber(port, type, avail) {
         if (!avail) {
             const num_p = Number(port);
@@ -385,60 +371,62 @@ class AddServicesModal extends React.Component {
     }
 
     validate(event) {
-        const ports = event.target.value.split(',');
         let error = "";
         let targets = ['tcp', 'custom_tcp_ports', 'tcp_error', 'custom_tcp_value'];
         if (event.target.id === "udp-ports")
             targets = ['udp', 'custom_udp_ports', 'udp_error', 'custom_udp_value'];
-
         const new_ports = [];
-        const self = this;
-        ports.forEach(function(port) {
-            port = port.trim();
-            if (!port)
-                return;
-            let ports;
-            if (port.indexOf("-") > -1) {
-                ports = port.split("-");
-                if (ports.length != 2) {
-                    error = _("Invalid range");
+        const event_value = event.target.value;
+        const event_id = event.target.id;
+
+        this.setState(oldState => {
+            const ports = event_value.split(',');
+            ports.forEach((port) => {
+                port = port.trim();
+                if (!port)
                     return;
-                }
-                [ports[0], error] = self.getPortNumber(ports[0], targets[0], self.state.avail_services[ports[0]]);
-                if (!error) {
-                    [ports[1], error] = self.getPortNumber(ports[1], targets[0], self.state.avail_services[ports[1]]);
-                    if (!error) {
-                        if (Number(ports[0]) >= Number(ports[1]))
-                            error = _("Range must be strictly ordered");
-                        else
-                            new_ports.push(ports[0] + "-" + ports[1]);
+                let ports;
+                if (port.indexOf("-") > -1) {
+                    ports = port.split("-");
+                    if (ports.length != 2) {
+                        error = _("Invalid range");
+                        return;
                     }
+                    [ports[0], error] = this.getPortNumber(ports[0], targets[0], oldState.avail_services[ports[0]]);
+                    if (!error) {
+                        [ports[1], error] = this.getPortNumber(ports[1], targets[0], oldState.avail_services[ports[1]]);
+                        if (!error) {
+                            if (Number(ports[0]) >= Number(ports[1]))
+                                error = _("Range must be strictly ordered");
+                            else
+                                new_ports.push(ports[0] + "-" + ports[1]);
+                        }
+                    }
+                } else {
+                    [ports, error] = this.getPortNumber(port, targets[0], oldState.avail_services[port]);
+                    if (!error)
+                        new_ports.push(ports);
                 }
-            } else {
-                [ports, error] = self.getPortNumber(port, targets[0], self.state.avail_services[port]);
-                if (!error)
-                    new_ports.push(ports);
+            });
+            const newState = {
+                [targets[1]]: new_ports,
+                [targets[2]]: error,
+                [targets[3]]: event_value
+            };
+
+            let all_ports = new_ports.concat(oldState.custom_udp_ports);
+            if (event_id === "udp-ports")
+                all_ports = oldState.custom_tcp_ports.concat(new_ports);
+
+            if (oldState.generate_custom_id) {
+                if (all_ports.length > 0)
+                    newState.custom_id = "custom--" + all_ports.map(this.getName).join('-');
+                else
+                    newState.custom_id = "";
             }
+
+            return newState;
         });
-        const newState = {
-            [targets[1]]: new_ports,
-            [targets[2]]: error,
-            [targets[3]]: event.target.value
-        };
-
-        let name = this.state.custom_name;
-        let all_ports = new_ports.concat(this.state.custom_udp_ports);
-        if (event.target.id === "udp-ports")
-            all_ports = this.state.custom_tcp_ports.concat(new_ports);
-
-        if (this.state.generate_name)
-            name = this.generateName(all_ports);
-
-        const new_id = "custom--" + all_ports.map(this.getName).join('-');
-
-        newState.custom_name = name;
-        newState.custom_id = new_id;
-        this.setState(newState);
     }
 
     onToggleType(event) {
@@ -449,12 +437,7 @@ class AddServicesModal extends React.Component {
 
     componentDidMount() {
         firewall.getAvailableServices()
-                .then(services => this.setState({
-                    services: services.map(s => {
-                        s.name = s.name || s.id;
-                        return s;
-                    })
-                }));
+                .then(services => this.setState({ services }));
         cockpit.file('/etc/services').read()
                 .done(content => this.setState({
                     avail_services: this.parseServices(content)
@@ -475,7 +458,7 @@ class AddServicesModal extends React.Component {
                 return false;
             });
         else if (this.state.filter && this.state.services)
-            services = this.state.services.filter(s => s.name.toLowerCase().indexOf(this.state.filter) > -1);
+            services = this.state.services.filter(s => s.id.indexOf(this.state.filter) > -1);
         else
             services = this.state.services;
 
@@ -520,7 +503,7 @@ class AddServicesModal extends React.Component {
                                                                                                 checked={this.state.selected.has(s.id)}
                                                                                                 onChange={this.onToggleService} /> }
                                                                         stacked
-                                                                        heading={ <label htmlFor={"firewall-service-" + s.id}>{s.name}</label> }
+                                                                        heading={ <label htmlFor={"firewall-service-" + s.id}>{s.id}</label> }
                                                                         description={ renderPorts(s) } />
                                                         ))
                                                     }
@@ -558,9 +541,9 @@ class AddServicesModal extends React.Component {
                                            placeholder={_("Example: 88,2019,nfs,rsync")} />
                                     <output className="has-error" htmlFor="udp-ports">{this.state.udp_error}</output>
 
-                                    <label className="control-label" htmlFor="service-name">Name</label>
-                                    <input id="service-name" className="form-control" type="text" onChange={this.setName}
-                                           placeholder={_("(Optional)")} value={this.state.custom_name} />
+                                    <label className="control-label" htmlFor="service-name">{_("Id")}</label>
+                                    <input id="service-name" className="form-control" type="text" onChange={this.setId}
+                                           placeholder={_("(Optional)")} value={this.state.custom_id} />
                                 </>
                             }
                         </form>
@@ -575,12 +558,12 @@ class AddServicesModal extends React.Component {
                             isInline
                             title={_("Adding custom ports will reload firewalld. A reload will result in the loss of any runtime-only configuration!")} />
                     }
-                    <Button variant='secondary' className='btn-cancel' onClick={this.props.close}>
-                        {_("Cancel")}
-                    </Button>
                     <Button variant='primary' onClick={this.save} aria-label={titleText}>
                         {addText}
                     </Button>
+                    <Button variant='link' className='btn-cancel' onClick={this.props.close}>
+                        {_("Cancel")}
+                    </Button>
                 </Modal.Footer>
             </Modal>
         );
@@ -668,7 +651,7 @@ class ActivateZoneModal extends React.Component {
                                 { zones.filter(z => firewall.predefinedZones.indexOf(z) !== -1).sort((a, b) => firewall.predefinedZones.indexOf(a) - firewall.predefinedZones.indexOf(b))
                                         .map(z =>
                                             <label className="radio" key={z}><input type="radio" name="zone" value={z} onChange={e => this.onChange("zone", e.target.value)} />
-                                                { firewall.zones[z].name || firewall.zones[z].id }
+                                                { firewall.zones[z].id }
                                             </label>
                                         )}
                             </fieldset>
@@ -676,7 +659,7 @@ class ActivateZoneModal extends React.Component {
                                 { customZones.length > 0 && <legend>{ _("Custom zones") }</legend> }
                                 { customZones.map(z =>
                                     <label className="radio" key={z}><input type="radio" name="zone" value={z} onChange={e => this.onChange("zone", e.target.value)} />
-                                        { firewall.zones[z].name || firewall.zones[z].id }
+                                        { firewall.zones[z].id }
                                     </label>
                                 )}
                             </fieldset>
@@ -726,15 +709,14 @@ class ActivateZoneModal extends React.Component {
                     {
                         this.state.dialogError && <ModalError dialogError={this.state.dialogError} dialogErrorDetail={this.state.dialogErrorDetail} />
                     }
-
-                    <Button variant="secondary" className="btn-cancel" onClick={this.props.close}>
-                        { _("Cancel") }
-                    </Button>
                     <Button variant="primary" onClick={this.save} isDisabled={this.state.zone === null ||
                                                                             (this.state.interfaces.size === 0 && this.state.ipRange === "ip-entire-subnet") ||
                                                                             (this.state.ipRange === "ip-range" && !this.state.ipRangeValue)}>
                         { _("Add zone") }
                     </Button>
+                    <Button variant="link" className="btn-cancel" onClick={this.props.close}>
+                        { _("Cancel") }
+                    </Button>
                 </Modal.Footer>
             </Modal>
         );
@@ -752,12 +734,12 @@ function DeleteConfirmationModal(props) {
                 <div>{props.body}</div>
             </Modal.Body>
             <Modal.Footer>
-                <Button variant="secondary" className="btn-cancel" onClick={props.onCancel}>
-                    { _("Cancel") }
-                </Button>
                 <Button variant="danger" onClick={props.onDelete} aria-label={cockpit.format(_("Confirm removal of $0"), props.target)}>
                     { _("Delete") }
                 </Button>
+                <Button variant="link" className="btn-cancel" onClick={props.onCancel}>
+                    { _("Cancel") }
+                </Button>
             </Modal.Footer>
         </Modal>
     );
@@ -879,28 +861,11 @@ export class Firewall extends React.Component {
                                     icon={ ExclamationCircleIcon } />;
         }
 
-        var addZoneAction;
-        if (this.state.firewall.readonly) {
-            addZoneAction = (
-                <OverlayTrigger className="pull-right" placement="top"
-                                overlay={ <Tooltip id="tip-auth">{ _("You are not authorized to modify the firewall.") }</Tooltip> }>
-                    <Button variant="primary" className="pull-right" id="add-zone-button" aria-label={_("Not authorized to add a new zone")} isDisabled> {_("Add Zone")} </Button>
-                </OverlayTrigger>
-            );
-        } else {
-            addZoneAction = (
-                <Button variant="primary" onClick={this.openAddZoneDialog} className="pull-right" id="add-zone-button" aria-label={_("Add a new zone")}>
-                    {_("Add Zone")}
-                </Button>
-            );
-        }
-
-        var services = [...this.state.firewall.enabledServices].map(id => {
-            const service = this.state.firewall.services[id];
-            service.name = service.name || id;
-            return service;
-        });
-        services.sort((a, b) => a.name.localeCompare(b.name));
+        var addZoneAction = (
+            <Button variant="primary" onClick={this.openAddZoneDialog} className="pull-right" id="add-zone-button" aria-label={_("Add a new zone")}>
+                {_("Add Zone")}
+            </Button>
+        );
 
         var zones = [...this.state.firewall.activeZones].sort((z1, z2) =>
             z1 === firewall.defaultZone ? -1 : z2 === firewall.defaultZone ? 1 : 0
@@ -910,13 +875,13 @@ export class Firewall extends React.Component {
 
         let firewallOnOff;
         if (firewall.readonly) {
-            firewallOnOff = <OverlayTrigger className="pull-right" placement="top"
-                                            overlay={ <Tooltip id="tip-auth">{ _("You are not authorized to modify the firewall.") }</Tooltip> }>
+            firewallOnOff = <Tooltip id="tip-auth"
+                                     content={ _("You are not authorized to modify the firewall.") }>
                 <OnOffSwitch state={enabled}
                              onChange={this.onSwitchChanged}
                              aria-label={enabled ? _("Not authorized to disable the firewall") : _("Not authorized to enable the firewall")}
                              disabled />
-            </OverlayTrigger>;
+            </Tooltip>;
         } else {
             firewallOnOff = <OnOffSwitch state={enabled}
                                          disabled={!!this.state.pendingTarget}
@@ -936,7 +901,7 @@ export class Firewall extends React.Component {
                             <h1>{_("Firewall")}</h1>
                             { firewallOnOff }
                         </span>
-                        { enabled && <span className="btn-group">{addZoneAction}</span> }
+                        { enabled && !firewall.readonly && <span className="btn-group">{addZoneAction}</span> }
                     </div>
                 </div>
                 <div id="zones-listing" className="container-fluid page-ct">
