@@ -27,10 +27,10 @@ import * as utils from "./utils.js";
 import React from "react";
 
 import { Listing, ListingRow } from "cockpit-components-listing.jsx";
-import { StorageButton, StorageLink } from "./storage-controls.jsx";
+import { StorageButton, StorageLink, StorageBarMenu, StorageMenuItem } from "./storage-controls.jsx";
 import { format_dialog } from "./format-dialog.jsx";
 
-import { FilesystemTab } from "./fsys-tab.jsx";
+import { FilesystemTab, is_mounted, mounting_dialog } from "./fsys-tab.jsx";
 import { CryptoTab } from "./crypto-tab.jsx";
 import { BlockVolTab, PoolVolTab } from "./lvol-tabs.jsx";
 import { PVolTab, MDRaidMemberTab, VDOBackingTab } from "./pvol-tabs.jsx";
@@ -68,8 +68,10 @@ function create_tabs(client, target, is_partition) {
     }
 
     var block = endsWith(target.iface, ".Block") ? target : null;
+    var block_fsys = block && client.blocks_fsys[block.path];
     var block_lvm2 = block && client.blocks_lvm2[block.path];
     var block_pvol = block && client.blocks_pvol[block.path];
+    var block_swap = block && client.blocks_swap[block.path];
 
     var lvol = (endsWith(target.iface, ".LogicalVolume")
         ? target
@@ -88,7 +90,7 @@ function create_tabs(client, target, is_partition) {
         if (associated_warnings)
             tab_warnings = warnings.filter(w => associated_warnings.indexOf(w.warning) >= 0);
         if (tab_warnings.length > 0)
-            name = <span><span className="pficon pficon-warning-triangle-o" /> {name}</span>;
+            name = <a><span className="pficon pficon-warning-triangle-o" /> {name}</a>;
         tabs.push(
             {
                 name: name,
@@ -145,8 +147,10 @@ function create_tabs(client, target, is_partition) {
         add_tab(_("Partition"), PartitionTab);
     }
 
+    let is_unrecognized = false;
+
     if (is_filesystem) {
-        add_tab(_("Filesystem"), FilesystemTab);
+        add_tab(_("Filesystem"), FilesystemTab, ["mismounted-fsys"]);
     } else if (is_crypto) {
         add_tab(_("Encryption"), CryptoTab);
     } else if ((block && block.IdUsage == "raid" && block.IdType == "LVM2_member") ||
@@ -160,13 +164,19 @@ function create_tabs(client, target, is_partition) {
     } else if (block && block.IdUsage == "other" && block.IdType == "swap") {
         add_tab(_("Swap"), SwapTab);
     } else if (block) {
+        is_unrecognized = true;
         add_tab(_("Unrecognized Data"), UnrecognizedTab);
     }
 
     var tab_actions = [];
+    var tab_menu_actions = [];
 
-    function add_action(title, func, excuse) {
-        tab_actions.push(<StorageButton key={title} onClick={func} excuse={excuse}>{title}</StorageButton>);
+    function add_action(title, func) {
+        tab_actions.push(<StorageButton key={title} onClick={func}>{title}</StorageButton>);
+    }
+
+    function add_menu_action(title, func) {
+        tab_menu_actions.push({ title: title, func: func });
     }
 
     function lock() {
@@ -240,7 +250,7 @@ function create_tabs(client, target, is_partition) {
 
     if (is_crypto) {
         if (client.blocks_cleartext[block.path]) {
-            add_action(_("Lock"), lock);
+            add_menu_action(_("Lock"), lock);
         } else {
             add_action(_("Unlock"), unlock);
         }
@@ -254,12 +264,46 @@ function create_tabs(client, target, is_partition) {
         return lvol.Deactivate({});
     }
 
+    function create_snapshot() {
+        dialog_open({
+            Title: _("Create Snapshot"),
+            Fields: [
+                TextInput("name", _("Name"),
+                          { validate: utils.validate_lvm2_name }),
+            ],
+            Action: {
+                Title: _("Create"),
+                action: function (vals) {
+                    return lvol.CreateSnapshot(vals.name, vals.size || 0, { });
+                }
+            }
+        });
+    }
+
     if (lvol) {
         if (lvol.Active) {
-            add_action(_("Deactivate"), deactivate);
+            add_menu_action(_("Deactivate"), deactivate);
         } else {
             add_action(_("Activate"), activate);
         }
+        if (client.lvols[lvol.ThinPool]) {
+            add_menu_action(_("Create Snapshot"), create_snapshot);
+        }
+    }
+
+    function swap_start() {
+        return block_swap.Start({});
+    }
+
+    function swap_stop() {
+        return block_swap.Stop({});
+    }
+
+    if (block_swap) {
+        if (block_swap.Active)
+            add_menu_action(_("Stop"), swap_stop);
+        else
+            add_menu_action(_("Start"), swap_start);
     }
 
     function delete_() {
@@ -313,19 +357,27 @@ function create_tabs(client, target, is_partition) {
     }
 
     if (is_partition || lvol) {
-        var excuse = null;
-        if (client.is_old_udisks2 && is_crypto && client.blocks_cleartext[block.path])
-            excuse = _("Can't delete while unlocked");
-        add_action(_("Delete"), delete_, excuse);
+        add_menu_action(_("Delete"), delete_);
     }
 
     if (block) {
-        add_action(_("Format"), () => format_dialog(client, block.path));
+        if (is_unrecognized)
+            add_action(_("Format"), () => format_dialog(client, block.path));
+        else
+            add_menu_action(_("Format"), () => format_dialog(client, block.path));
+    }
+
+    if (block_fsys) {
+        if (is_mounted(client, block))
+            add_menu_action(_("Unmount"), () => mounting_dialog(client, block, "unmount"));
+        else
+            add_action(_("Mount"), () => mounting_dialog(client, block, "mount"));
     }
 
     return {
         renderers: tabs,
-        actions: <>{tab_actions}</>,
+        actions: tab_actions,
+        menu_actions: tab_menu_actions,
         row_action: row_action,
         has_warnings: warnings.length > 0
     };
@@ -392,18 +444,28 @@ function append_row(client, rows, level, key, name, desc, tabs, job_object) {
     }
 
     var cols = [
-        <span className={"content-level-" + level}>
+        <span key={name} className={"content-level-" + level}>
             {utils.format_size_and_text(desc.size, desc.text)}
         </span>,
         { name: name, header: true },
         { name: last_column, tight: true },
     ];
 
+    function menuitem(action) {
+        return <StorageMenuItem key={action.title} onClick={action.func}>{action.title}</StorageMenuItem>;
+    }
+
+    var menu = null;
+    if (tabs.menu_actions && tabs.menu_actions.length > 0)
+        menu = <StorageBarMenu id={"menu-" + name}>{tabs.menu_actions.map(menuitem)}</StorageBarMenu>;
+
+    var actions = <>{tabs.actions}{menu}</>;
+
     rows.push(
         <ListingRow key={key}
                     columns={cols}
                     tabRenderers={tabs.renderers}
-                    listingActions={tabs.actions} />
+                    listingActions={actions} />
     );
 }
 
@@ -441,7 +503,7 @@ function append_partitions(client, rows, level, block) {
         );
 
         var cols = [
-            <span className={"content-level-" + level}>
+            <span key={start.toString() + size.toString()} className={"content-level-" + level}>
                 {utils.format_size_and_text(size, _("Free Space"))}
             </span>,
             "",
