@@ -17,9 +17,24 @@
  * along with Cockpit; If not, see <http://www.gnu.org/licenses/>.
  */
 
+import '../../src/base1/patternfly-cockpit.scss';
+
 import $ from "jquery";
 import cockpit from "cockpit";
 import { journal } from "journal";
+import moment from "moment";
+import { init_reporting } from "./reporting.jsx";
+import { superuser } from "superuser.jsx";
+
+import ReactDOM from 'react-dom';
+import React from 'react';
+import { EmptyStatePanel } from "cockpit-components-empty-state.jsx";
+import { ExclamationCircleIcon } from '@patternfly/react-icons';
+
+// We open a couple of long-running channels with { superuser: "try" },
+// so we need to reload the page if the access level changes.
+//
+superuser.reload_page_on_change();
 
 $(function() {
     cockpit.translate();
@@ -27,6 +42,7 @@ $(function() {
 
     var update_services_list = true;
     var current_services = new Set();
+    let the_journal = null;
 
     var problems_client = cockpit.dbus('org.freedesktop.problems', { superuser: "try" });
     var service = problems_client.proxy('org.freedesktop.Problems2', '/org/freedesktop/Problems2');
@@ -93,10 +109,23 @@ $(function() {
         }
     }
 
+    function manage_start_box(loading, show_icon, title, text, action, onAction) {
+        ReactDOM.render(
+            React.createElement(EmptyStatePanel, {
+                loading: loading,
+                icon: show_icon ? ExclamationCircleIcon : undefined,
+                title: title,
+                paragraph: text,
+                action: action,
+                onAction: onAction,
+            }),
+            document.getElementById("start-box"));
+    }
+
     /* Not public API */
-    function journalbox(outer, start, match, day_box) {
+    function journalbox(outer, start, match, priority, tag, keep_following, grep) {
         var box = $('<div class="panel panel-default cockpit-log-panel" role="table">');
-        var start_box = $('<div class="journal-start" role="rowgroup">');
+        var start_box = $('<div class="journal-start" id="start-box" role="rowgroup">');
 
         outer.empty().append(box, start_box);
 
@@ -104,116 +133,92 @@ $(function() {
         var query_more = 1000;
 
         var renderer = journal.renderer(box);
-        /* cache to store offsets for days */
-        var renderitems_day_cache = null;
         var procs = [];
+        var following_procs = [];
 
         var loading_services = false;
 
+        let no_logs = true;
+
         function query_error(error) {
             /* TODO: blank slate */
-            console.warn(cockpit.message(error));
+            console.error(cockpit.message(error));
         }
 
         function prepend_entries(entries) {
+            if (entries.length > 0)
+                no_logs = false;
             for (var i = 0; i < entries.length; i++) {
                 renderer.prepend(entries[i]);
                 current_services.add(entries[i].SYSLOG_IDENTIFIER);
             }
             renderer.prepend_flush();
             show_service_filters();
-            /* empty cache for day offsets */
-            renderitems_day_cache = null;
         }
 
         function append_entries(entries) {
+            if (entries.length > 0)
+                no_logs = false;
             for (var i = 0; i < entries.length; i++)
                 renderer.append(entries[i]);
             renderer.append_flush();
-            /* empty cache for day offsets */
-            renderitems_day_cache = null;
         }
 
         function didnt_reach_start(first) {
-            var button = $('<button id="journal-load-earlier" class="btn btn-default" data-inline="true" data-mini="true">' +
-                           _("Load earlier entries") +
-                           '</button>');
-            start_box.html(button);
-            button.click(function() {
-                var count = 0;
-                var stopped = null;
-                start_box.text(_("Loading..."));
-                procs.push(journal.journalctl(match, { follow: false, reverse: true, cursor: first })
-                        .fail(query_error)
-                        .stream(function(entries) {
-                            if (entries[0].__CURSOR == first)
-                                entries.shift();
-                            count += entries.length;
-                            append_entries(entries);
-                            if (count >= query_more) {
-                                stopped = entries[entries.length - 1].__CURSOR;
-                                didnt_reach_start(stopped);
-                                this.stop();
-                            }
-                        })
-                        .done(function() {
-                            if (start_box.text() == _("Loading..."))
-                                start_box.empty();
-                        }));
-            });
+            manage_start_box(false, no_logs,
+                             no_logs ? _("No Logs Found") : "",
+                             no_logs ? _("You may try to load older entries.") : "",
+                             _("Load earlier entries"),
+                             () => {
+                                 var count = 0;
+                                 var stopped = null;
+                                 manage_start_box(true, false, no_logs ? ("Loading...") : null, "", "");
+                                 procs.push(journal.journalctl(match, { follow: false, reverse: true, cursor: first, priority: priority, grep: grep })
+                                         .fail(query_error)
+                                         .stream(function(entries) {
+                                             if (entries[0].__CURSOR == first)
+                                                 entries.shift();
+                                             count += entries.length;
+                                             append_entries(entries);
+                                             if (count >= query_more) {
+                                                 stopped = entries[entries.length - 1].__CURSOR;
+                                                 didnt_reach_start(stopped);
+                                                 this.stop();
+                                             }
+                                         })
+                                         .done(function() {
+                                             if (no_logs)
+                                                 manage_start_box(false, true, _("No Logs Found"), _("Can not find any logs using the current combination of filters."));
+                                             else if (count < query_more)
+                                                 ReactDOM.unmountComponentAtNode(document.getElementById("start-box"));
+                                         }));
+                             });
         }
 
         function follow(cursor) {
-            procs.push(journal.journalctl(match, { follow: true, count: 0, cursor: cursor })
+            following_procs.push(journal.journalctl(match, { follow: true, count: 0, cursor: cursor || null, priority: priority, grep: grep })
                     .fail(query_error)
                     .stream(function(entries) {
                         if (entries[0].__CURSOR == cursor)
                             entries.shift();
+                        if (entries.length > 0 && no_logs)
+                            ReactDOM.unmountComponentAtNode(document.getElementById("start-box"));
                         prepend_entries(entries);
-                        update_day_box();
                     }));
         }
-
-        function update_day_box() {
-            /* Build cache if empty
-             */
-            if (renderitems_day_cache === null) {
-                renderitems_day_cache = [];
-                for (var d = box[0].firstChild; d; d = d.nextSibling) {
-                    if ($(d).hasClass('panel-heading'))
-                        renderitems_day_cache.push([$(d).offset().top, $(d).text()]);
-                }
-            }
-            if (renderitems_day_cache.length > 0) {
-                /* Find the last day that begins above top
-                 */
-                var currentIndex = 0;
-                var top = window.scrollY;
-                while ((currentIndex + 1) < renderitems_day_cache.length &&
-                        renderitems_day_cache[currentIndex + 1][0] < top) {
-                    currentIndex++;
-                }
-                day_box.text(renderitems_day_cache[currentIndex][1]);
-            } else {
-                /* No visible day headers
-                 */
-                day_box.text(_("Go to"));
-            }
-        }
+        outer.follow = follow;
 
         function clear_service_list() {
             if (loading_services) {
-                $('#journal-services-list').empty()
-                        .append($('<li>').text(_("Loading...")));
+                $('#journal-service-menu').empty()
+                        .append($('<option selected disabled>').text(_("Loading...")));
                 return;
             }
 
-            $('#journal-services-list').empty()
-                    .append($('<li>').append($('<a>')
-                            .text(_("All"))
-                            .attr('data-service', "")))
-                    .append($('<li>')
-                            .addClass("divider"));
+            $('#journal-service-menu').empty()
+                    .append($('<option value="*">').text(_("All")))
+                    .append($('<option value="" role="separator" className="divider" disabled>').text("──────────"));
+            fit_filters();
         }
 
         function load_service_filters(match, options) {
@@ -245,29 +250,19 @@ $(function() {
             Array.from(current_services).sort((a, b) =>
                 a.toLowerCase().localeCompare(b.toLowerCase())
             )
-                    .forEach(function(unit) {
-                        $('#journal-services-list').append(
-                            $('<li>').append($('<a>')
-                                    .text(unit)
-                                    .attr('data-service', unit)));
-                    });
+                    .forEach(unit => $('#journal-service-menu').append($('<option value="' + unit + '"' + (unit === tag ? ' selected>' : '>')).text(unit)));
         }
 
-        start_box.text(_("Loading..."));
-
-        $('#journal-service-menu').on("click", "a", function() {
-            update_services_list = false;
-            cockpit.location.go([], $.extend(cockpit.location.options, { tag: $(this).attr('data-service') }));
-        });
-
-        $(window).on('scroll', update_day_box);
+        manage_start_box(true, false, _("Loading..."), "", "");
 
         var options = {
             follow: false,
-            reverse: true
+            reverse: true,
+            priority: priority,
+            grep: grep,
         };
 
-        var last = null;
+        let last = keep_following ? null : 1;
         var count = 0;
         var oldest = null;
         var stopped = false;
@@ -277,7 +272,6 @@ $(function() {
             options.boot = null;
         } else if (start == 'previous-boot') {
             options.boot = "-1";
-            last = 1; // Do not try to get newer logs
         } else if (start == 'last-24h') {
             options.since = "-1days";
         } else if (start == 'last-week') {
@@ -297,13 +291,20 @@ $(function() {
             load_service_filters(tags_match, options);
         }
 
+        // Show the journalctl query in inline help
+        const cmd = journal.build_cmd(match, options);
+        const filtered_cmd = cmd[0].filter(i => i !== "-q" && i !== "--output=json");
+        if (filtered_cmd[filtered_cmd.length - 1] == "--")
+            filtered_cmd.pop();
+
+        document.getElementById("journal-query").innerHTML = filtered_cmd.join(" ");
+
         procs.push(journal.journalctl(match, options)
                 .fail(query_error)
                 .stream(function(entries) {
                     if (!last) {
                         last = entries[0].__CURSOR;
                         follow(last);
-                        update_day_box();
                     }
                     count += entries.length;
                     append_entries(entries);
@@ -315,18 +316,23 @@ $(function() {
                     }
                 })
                 .done(function() {
-                    if (start_box.text() == _("Loading..."))
-                        start_box.empty();
+                    if (no_logs)
+                        manage_start_box(false, true, _("No Logs Found"), _("Can not find any logs using the current combination of filters."));
+                    else if (count < query_count)
+                        ReactDOM.unmountComponentAtNode(document.getElementById("start-box"));
                     if (!last) {
-                        procs.push(journal.journalctl(match, {
+                        following_procs.push(journal.journalctl(match, {
                             follow: true, count: 0,
                             boot: options.boot,
-                            since: options.since
+                            since: options.since,
+                            priority: priority,
+                            grep: grep,
                         })
                                 .fail(query_error)
                                 .stream(function(entries) {
+                                    if (entries.length > 0 && no_logs)
+                                        ReactDOM.unmountComponentAtNode(document.getElementById("start-box"));
                                     prepend_entries(entries);
-                                    update_day_box();
                                 }));
                     }
                     if (!all || stopped)
@@ -334,10 +340,18 @@ $(function() {
                 }));
 
         outer.stop = function stop() {
-            $(window).off('scroll', update_day_box);
             $.each(procs, function(i, proc) {
                 proc.stop();
             });
+            $.each(following_procs, function(i, proc) {
+                proc.stop();
+            });
+        };
+
+        outer.stop_following = function stop_following() {
+            $.each(following_procs, function(i, proc) {
+                proc.stop();
+            });
         };
 
         return outer;
@@ -354,52 +368,94 @@ $(function() {
         stop_query();
 
         var match = [];
+        const options = cockpit.location.options;
 
-        var query_prio = cockpit.location.options.prio || "3";
-        var prio_level = parseInt(query_prio, 10);
+        const grep = options.grep || "";
+        let full_grep = "";
 
-        // Set selected item into priority dropdown menu
-        var all_prios = document.getElementById('prio-lists').childNodes;
-        var item;
-        for (var j = 0; j < all_prios.length; j++) {
-            if (all_prios[j].nodeName === 'LI') {
-                item = all_prios[j].childNodes[0];
-                if (item.getAttribute('data-prio') === query_prio) {
-                    $('#journal-prio').text(item.text);
-                    break;
-                }
-            }
-        }
-
-        if (!isNaN(prio_level)) {
-            for (var i = 0; i <= prio_level; i++)
-                match.push('PRIORITY=' + i.toString());
-        }
-
-        var options = cockpit.location.options;
-        if (options.service)
-            match.push('_SYSTEMD_UNIT=' + options.service);
-        else if (options.tag)
-            match.push('SYSLOG_IDENTIFIER=' + options.tag);
-        $('#journal-service').text(options.tag || _("All"));
-
-        var query_start = cockpit.location.options.start || "recent";
+        // Set selected item into start time select menu
+        var query_start = options.start || "recent";
         if (query_start == 'recent')
             $(window).scrollTop($(document).height());
+        else
+            full_grep += "start:" + query_start + " ";
 
-        journalbox($("#journal-box"), query_start, match, $('#journal-current-day'));
+        const prio_level = options.prio || "err";
+        full_grep += "priority:" + prio_level + " ";
+
+        // Set selected item into priority select menu
+        const prio_options = [...document.getElementById('journal-prio-menu').children];
+        prio_options.forEach(p => {
+            if (p.getAttribute('value') === prio_level)
+                p.selected = true;
+            else
+                p.selected = false;
+        });
+
+        const time_options = [...document.getElementById('journal-current-day-menu').children];
+        time_options.forEach(p => {
+            if (p.getAttribute('value') === query_start)
+                p.selected = true;
+            else
+                p.selected = false;
+        });
+
+        let follow = !(options.follow && options.follow === "false");
+
+        if (query_start == "previous-boot") // Don't follow if we want to see only previous boot
+            follow = false;
+
+        const follow_button = document.getElementById("journal-follow");
+        if (follow) {
+            follow_button.textContent = _("Pause");
+            follow_button.setAttribute("data-following", true);
+        } else {
+            follow_button.textContent = _("Resume");
+            follow_button.setAttribute("data-following", false);
+        }
+
+        if (options.service) {
+            let s = options.service;
+            if (!s.endsWith(".service"))
+                s = s + ".service";
+            match.push(...['_SYSTEMD_UNIT=' + s, "+", "COREDUMP_UNIT=" + s, "+", "UNIT=" + s]);
+            full_grep += "service:" + options.service + " ";
+        }
+
+        if (options.tag && options.tag !== "*") {
+            match.push('SYSLOG_IDENTIFIER=' + options.tag);
+            full_grep += "identifier:" + options.tag + " ";
+        }
+
+        // Other filters may be passed as well
+        Object.keys(options).forEach(k => {
+            if (k === k.toUpperCase() && options[k]) {
+                options[k].split(",").forEach(v => match.push(k + "=" + v));
+                full_grep += k + '=' + options[k] + " ";
+            }
+        });
+
+        full_grep += grep;
+        document.getElementById("journal-grep").value = full_grep;
+
+        if (the_journal)
+            the_journal.stop();
+
+        the_journal = journalbox($("#journal-box"), query_start, match, prio_level, options.tag, follow, grep);
     }
 
     function update_entry() {
         var cursor = cockpit.location.path[0];
         var out = $('#journal-entry-fields');
 
+        const reportingTable = document.getElementById("journal-entry-reporting-table");
+        if (reportingTable != null) {
+            reportingTable.remove();
+        }
+
         out.empty();
 
         function show_entry(entry) {
-            var d = new Date(entry.__REALTIME_TIMESTAMP / 1000);
-            $('#journal-entry-date').text(d.toString());
-
             var id;
             if (entry.SYSLOG_IDENTIFIER)
                 id = entry.SYSLOG_IDENTIFIER;
@@ -414,13 +470,20 @@ $(function() {
                 id = entry.PROBLEM_BINARY;
             }
 
-            $('#journal-entry-id').text(id);
+            $('#journal-entry-heading').text(id);
+
+            const crumb = $("#journal-entry-crumb");
+            const date = moment(new Date(entry.__REALTIME_TIMESTAMP / 1000));
 
             if (is_problem) {
+                crumb.text(cockpit.format(_("$0: crash at $1"), id, date.format("YYYY-MM-DD HH:mm:ss")));
+
                 find_problems().done(function() {
                     create_problem(out, entry);
                 });
             } else {
+                crumb.text(cockpit.format(_("Entry at $0"), date.format("YYYY-MM-DD HH:mm:ss")));
+
                 create_entry(out, entry);
             }
         }
@@ -444,17 +507,36 @@ $(function() {
                 });
     }
 
+    function create_message_row(entry) {
+        const reasonColumn = document.createElement("th");
+        reasonColumn.setAttribute("colspan", 2);
+        reasonColumn.setAttribute("id", "journal-entry-message");
+        reasonColumn.appendChild(document.createTextNode(journal.printable(entry.MESSAGE)));
+
+        const reason = document.createElement("tr");
+        reason.appendChild(reasonColumn);
+
+        return reason;
+    }
+
     function create_entry(out, entry) {
-        $('#journal-entry-message').text(journal.printable(entry.MESSAGE));
+        out.append(create_message_row(entry));
+
         var keys = Object.keys(entry).sort();
         $.each(keys, function (i, key) {
             if (key !== 'MESSAGE') {
                 out.append(
                     $('<tr>').append(
-                        $('<td>').css('text-align', 'right')
-                                .text(key),
-                        $('<td>').css('text-align', 'left')
-                                .text(journal.printable(entry[key]))));
+                        $('<td>', {
+                            class: "journal-entry-key",
+                            text: key
+                        }),
+                        $('<td>', {
+                            class: "journal-entry-value",
+                            text: journal.printable(entry[key])
+                        })
+                    )
+                );
             }
         });
     }
@@ -483,11 +565,15 @@ $(function() {
                     .replaceWith(new_content);
         }
 
-        $('#journal-entry-message').text('');
+        const heading = document.createElement("h3");
+        heading.appendChild(document.createTextNode(_("Extended Information")));
 
-        var ge_t = $('<li class="active">').append($('<a tabindex="0">').append($('<span translatable="yes">').text(_("General"))));
-        var pi_t = $('<li>').append($('<a tabindex="0">').append($('<span translatable="yes">').text(_("Problem info"))));
-        var pd_t = $('<li>').append($('<a tabindex="0">').append($('<span translatable="yes">').text(_("Problem details"))));
+        const caption = document.createElement("caption");
+        caption.appendChild(heading);
+
+        var ge_t = $('<li class="active">').append($('<a tabindex="0">').append($('<span translate="yes">').text(_("General"))));
+        var pi_t = $('<li>').append($('<a tabindex="0">').append($('<span translate="yes">').text(_("Problem info"))));
+        var pd_t = $('<li>').append($('<a tabindex="0">').append($('<span translate="yes">').text(_("Problem details"))));
 
         var ge = $('<tbody>').addClass('tab');
         var pi = $('<tbody>').addClass('tab');
@@ -495,51 +581,18 @@ $(function() {
                 .append(
                     $('<tr>').append($('<div class="panel-group" id="accordion-markup">')));
 
-        var tab = $('<ul class="nav nav-tabs nav-tabs-pf">');
+        var tab = $('<ul class="nav nav-tabs nav-tabs-pf">')
+                .attr("id", "problem-navigation");
 
-        var d_btn = $('<button class="btn btn-danger problem-btn btn-delete pficon pficon-delete">');
-        var r_btn = $();
-        if (problem.IsReported) {
-            for (var pid = 0; pid < problem.Reports.length; pid++) {
-                if (problem.Reports[pid][0] === 'ABRT Server') {
-                    var url = problem.Reports[pid][1].URL.v.v;
-                    r_btn = $('<a class="problem-btn">')
-                            .attr('href', url)
-                            .attr("target", "_blank")
-                            .text(_("Reported"));
-                    break;
-                }
-            }
-        } else if (problem.CanBeReported) {
-            r_btn = $('<button class="btn btn-primary problem-btn">').text(_("Report"));
+        var d_btn = $('<button class="pf-c-button pf-m-danger problem-btn btn-delete">').append('<span class="pficon pficon-delete">');
 
-            r_btn.click(function() {
-                tab.children(':last-child').replaceWith($('<div class="spinner problem-btn">'));
-                var proc = cockpit.spawn(['reporter-ureport', '-d', problem.ID], { superuser: 'true' });
-                proc.done(function() {
-                    window.location.reload();
-                });
-                proc.fail(function(ex) {
-                    var message;
-                    // 70 is 'This problem has already been reported'
-                    if (ex.exit_status === 70) {
-                        window.location.reload();
-                        return;
-                    } else if (ex.problem === 'access-denied') {
-                        message = _("Not authorized to upload-report");
-                    } else if (ex.problem === "not-found") {
-                        message = _("Reporter 'reporter-ureport' not found.");
-                    } else {
-                        message = _("Reporting was unsucessful. Try running `reporter-ureport -d " + problem.ID + "`");
-                    }
-                    $('<div class="alert alert-danger">')
-                            .append('<span class="pficon pficon-error-circle-o">')
-                            .text(message)
-                            .insertAfter(".breadcrumb");
-                    tab.children(':last-child').replaceWith($('<span>'));
-                });
-            });
-        }
+        const reportingTable = document.createElement("div");
+        reportingTable.setAttribute("id", "journal-entry-reporting-table");
+
+        const journalTable = document.getElementById("journal-entry-fields");
+        journalTable.insertAdjacentElement("beforebegin", reportingTable);
+
+        init_reporting(problem, reportingTable);
 
         ge_t.click(function() {
             switch_tab(ge_t, ge);
@@ -578,13 +631,15 @@ $(function() {
         tab.append(pi_t);
         tab.append(pd_t);
         tab.append(d_btn);
-        tab.append(r_btn);
 
         var header = $('<tr>').append(
             $('<th colspan=2>').append(tab));
 
-        out.html(header).append(ge);
+        out.html(header).append(create_message_row(entry));
+        out.append(ge);
+        out.prepend(caption);
         out.css("margin-bottom", "0px");
+
         create_problem_details(problem, pi, pd);
     }
 
@@ -771,7 +826,7 @@ $(function() {
         detail_content += create_table_from_thread(crash_thread);
 
         if (other_threads.length !== 0) {
-            detail_content += '<div id="other_threads_btn_div"><button class="btn btn-default other-threads-btn" title="">Show all threads</button></div>';
+            detail_content += '<div id="other_threads_btn_div"><button class="pf-c-button pf-m-secondary other-threads-btn" title="">Show all threads</button></div>';
             detail_content += '<div class="hidden other_threads">';
 
             var thread_num = 1;
@@ -866,14 +921,14 @@ $(function() {
     function update() {
         var path = cockpit.location.path;
         if (path.length === 0) {
-            $("#journal-entry").hide();
+            $("#journal-entry").prop("hidden", true);
             update_query();
             $("#journal").show();
         } else if (path.length == 1) {
             stop_query();
             $("#journal").hide();
             update_entry();
-            $("#journal-entry").show();
+            $("#journal-entry").prop("hidden", false);
         } else { /* redirect */
             console.warn("not a journal location: " + path);
             cockpit.location = '';
@@ -881,25 +936,136 @@ $(function() {
         $("body").show();
     }
 
+    function check_grep() {
+        // Sometimes `journalctl` can be compiled without `PCRE2` which means
+        // that `--grep` is not usable. Hide the search box in such cases.
+        cockpit.spawn(["journalctl", "--version"])
+                .then(m => {
+                    if (m.indexOf("-PCRE2") !== -1) {
+                        document.querySelector(".text-search").hidden = true;
+                        fit_filters();
+                    }
+                });
+    }
+
+    function fit_filters() {
+        if ($(".filters-toggle button").is(":hidden")) {
+            $("#journal .content-header-extra").toggleClass("toggle-filters-closed");
+            $(".filters-toggle").prop("hidden", false);
+            $(".filters-toggle button").text(_("Show filters"));
+        } else {
+            $(".filters-toggle button").text(_("Hide filters"));
+        }
+    }
+
+    function parse_search(value) {
+        const new_items = {};
+        const values = value.split(" ")
+                .filter(item => {
+                    let s = item.split("=");
+                    if (s.length === 2 && s[0] === s[0].toUpperCase()) {
+                        new_items[s[0]] = s[1];
+                        return false;
+                    }
+
+                    const well_know_keys = ["start", "priority", "follow", "service", "identifier"];
+                    const map_keys = (key) => {
+                        if (key === "priority")
+                            return "prio";
+                        if (key === "identifier")
+                            return "tag";
+                        return key;
+                    };
+                    s = item.split(":");
+                    if (s.length === 2 && well_know_keys.includes(s[0])) {
+                        new_items[map_keys(s[0])] = s[1];
+                        return false;
+                    }
+
+                    return true;
+                });
+        new_items.grep = values.join(" ");
+        return new_items;
+    }
+
     $(cockpit).on("locationchanged", function() {
         update_services_list = true;
         update();
     });
 
-    $('#journal-current-day-menu a').on('click', function() {
-        update_services_list = true;
-        cockpit.location.go([], $.extend(cockpit.location.options, { start: $(this).attr("data-op") }));
-    });
-
     $('#journal-box').on('click', '.cockpit-logline', function() {
         var cursor = $(this).attr('data-cursor');
         if (cursor)
             cockpit.location.go([cursor], { parent_options: JSON.stringify(cockpit.location.options) });
     });
 
-    $('#journal-prio-menu a').on('click', function() {
+    $('#journal-box').on('keypress', '.cockpit-logline', function(ev) {
+        if (ev.key !== "Enter")
+            return;
+
+        var cursor = $(this).attr('data-cursor');
+        if (cursor)
+            cockpit.location.go([cursor], { parent_options: JSON.stringify(cockpit.location.options) });
+    });
+
+    $('#journal-cmd-copy').on('click', function(ev) {
+        ev.preventDefault();
+        ev.stopPropagation();
+        try {
+            navigator.clipboard.writeText(document.getElementById("journal-query").innerHTML)
+                    .then(() => {
+                        let icon = ev.target;
+                        if (ev.target.nodeName == "BUTTON")
+                            icon = ev.target.firstElementChild;
+                        icon.classList.remove("fa-clipboard");
+                        icon.classList.add("fa-check");
+                        icon.classList.add("green-icon");
+                        setTimeout(() => {
+                            icon.classList.remove("fa-check");
+                            icon.classList.remove("green-icon");
+                            icon.classList.add("fa-clipboard");
+                        }, 3000);
+                    })
+                    .catch(e => console.error('Text could not be copied: ', e ? e.toString() : ""));
+        } catch (error) {
+            console.error('Text could not be copied: ', error.toString());
+        }
+    });
+
+    $('#journal-current-day-menu').on('change', function() {
+        const options = parse_search(document.getElementById("journal-grep").value);
         update_services_list = true;
-        cockpit.location.go([], $.extend(cockpit.location.options, { prio: $(this).attr('data-prio') }));
+        cockpit.location.go([], $.extend(options, { start: $(this).val() }));
+    });
+
+    $('#journal-prio-menu').on('change', function() {
+        const options = parse_search(document.getElementById("journal-grep").value);
+        update_services_list = true;
+        cockpit.location.go([], $.extend(options, { prio: $(this).val() }));
+    });
+
+    $('#journal-service-menu').on("change", function() {
+        const options = parse_search(document.getElementById("journal-grep").value);
+        update_services_list = false;
+        cockpit.location.go([], $.extend(options, { tag: $(this).val() }));
+    });
+
+    $('#journal-follow').on("click", function() {
+        const state = $(this).attr("data-following") === "true";
+        if (state) {
+            $(this).text(_("Resume"));
+            $(this).attr("data-following", false);
+            the_journal.stop_following();
+        } else {
+            $(this).text(_("Pause"));
+            $(this).attr("data-following", true);
+
+            const cursor = document.querySelector(".cockpit-logline");
+            if (cursor)
+                the_journal.follow(cursor.getAttribute("data-cursor"));
+            else
+                the_journal.follow();
+        }
     });
 
     $('#journal-navigate-home').on("click", function() {
@@ -915,5 +1081,25 @@ $(function() {
         cockpit.location.go('/', parent_options);
     });
 
+    $(".filters-toggle button").on("click", () => {
+        if ($("#journal .content-header-extra").hasClass("toggle-filters-closed"))
+            $(".filters-toggle button").text(_("Hide filters"));
+        else
+            $(".filters-toggle button").text(_("Show filters"));
+        $("#journal .content-header-extra").toggleClass("toggle-filters-closed");
+    });
+
+    $('#journal-grep').on("keyup", function(e) {
+        if (e.keyCode == 13) { // Submitted by enter
+            update_services_list = true;
+            cockpit.location.go([], parse_search($(this).val()));
+        }
+    });
+
+    // Check if the last filter is still on the same line, or it was wrapped.
+    window.addEventListener("resize", fit_filters);
+
+    check_grep();
+    fit_filters();
     update();
 });
