@@ -54,7 +54,6 @@ __all__ = (
     'nondestructive',
     'skipImage',
     'skipBrowser',
-    'allowImage',
     'skipPackage',
     'enableAxe',
     'timeout',
@@ -170,6 +169,12 @@ class Browser:
             msg += "\n" + trailer
         raise Error("%s(%s): %s" % (func, arg, msg))
 
+    # Execute js code that does not return anything
+    def inject_js(self, code):
+        self.cdp.invoke("Runtime.evaluate", expression=code, trace=code,
+                        silent=False, awaitPromise=True, returnByValue=False, no_trace=True)
+
+    # Execute js code that returns something
     def eval_js(self, code, no_trace=False):
         result = self.cdp.invoke("Runtime.evaluate", expression=code, trace=code,
                                  silent=False, awaitPromise=True, returnByValue=True, no_trace=no_trace)
@@ -519,11 +524,24 @@ class Browser:
         else:
             self.click(sel + ' button:first-child')
 
-    def login_and_go(self, path=None, user=None, host=None, authorized=True, urlroot=None, tls=False, password=None):
+    def try_login(self, user=None, password=None, authorized=True, superuser=True):
+        """Fills in the login dialog and clicks the button.
+
+        This differs from login_and_go() by not expecting any particular result.
+        """
         if user is None:
             user = self.default_user
         if password is None:
             password = self.password
+        self.wait_visible("#login")
+        self.set_val('#login-user-input', user)
+        self.set_val('#login-password-input', password)
+        self.set_checked('#authorized-input', authorized)
+        if superuser is not None:
+            self.eval_js('window.localStorage.setItem("superuser:%s", "%s");' % (user, "any" if superuser else "none"))
+        self.click('#login-button')
+
+    def login_and_go(self, path=None, user=None, host=None, authorized=True, superuser=True, urlroot=None, tls=False, password=None):
         href = path
         if not href:
             href = "/"
@@ -532,12 +550,9 @@ class Browser:
         if host:
             href = "/@" + host + href
         self.open(href, tls=tls)
-        self.wait_visible("#login")
-        self.set_val('#login-user-input', user)
-        self.set_val('#login-password-input', password)
-        self.set_checked('#authorized-input', authorized)
 
-        self.click('#login-button')
+        self.try_login(user, password, authorized=authorized, superuser=superuser)
+
         self.expect_load()
         self.wait_present('#content')
         self.wait_visible('#content')
@@ -557,7 +572,7 @@ class Browser:
             self.click('#go-logout')
         self.expect_load()
 
-    def relogin(self, path=None, user=None, authorized=None):
+    def relogin(self, path=None, user=None, authorized=None, superuser=None):
         if user is None:
             user = self.default_user
         self.logout()
@@ -566,6 +581,8 @@ class Browser:
         self.set_val("#login-password-input", self.password)
         if authorized is not None:
             self.set_checked('#authorized-input', authorized)
+        if superuser is not None:
+            self.eval_js('window.localStorage.setItem("superuser:%s", "%s");' % (user, "any" if superuser else "none"))
         self.click('#login-button')
         self.expect_load()
         self.wait_present('#content')
@@ -646,6 +663,37 @@ class Browser:
         self.cdp.kill()
 
 
+class _DebugOutcome(unittest.case._Outcome):
+    '''Run debug actions after test methods
+
+    This will do screenshots, HTML dumps, and sitting before cleanup handlers run.
+    '''
+    def testPartExecutor(self, test_case, isTest=False):
+        # we want to do the debug actions after the test function got called (isTest==True), but before
+        # TestCase.run() calls tearDown(); thus remember whether we are after isTest and already ran
+        if isTest:
+            self.ran_debug = False
+        else:
+            if getattr(self, "ran_debug", None) is False:
+                self.ran_debug = True
+                (err_case, exc_info) = self.errors[-1]
+                if exc_info and isinstance(test_case, MachineCase):
+                    assert err_case == test_case
+                    # strip off the two topmost frames for testPartExecutor and TestCase.run(); uninteresting and breaks naughties
+                    traceback.print_exception(exc_info[0], exc_info[1], exc_info[2].tb_next.tb_next)
+                    test_case.snapshot("FAIL")
+                    test_case.copy_js_log("FAIL")
+                    test_case.copy_journal("FAIL")
+                    test_case.copy_cores("FAIL")
+                    if opts.sit:
+                        sit(test_case.machines)
+
+        return super().testPartExecutor(test_case, isTest)
+
+
+unittest.case._Outcome = _DebugOutcome
+
+
 class MachineCase(unittest.TestCase):
     image = testvm.DEFAULT_IMAGE
     runner = None
@@ -663,9 +711,9 @@ class MachineCase(unittest.TestCase):
     provision = None
 
     @classmethod
-    def get_global_machine(klass):
+    def get_global_machine(klass, restrict=True):
         if not klass.global_machine:
-            klass.global_machine = klass.new_machine(klass, cleanup=False)
+            klass.global_machine = klass.new_machine(klass, restrict=restrict, cleanup=False)
             if opts.trace:
                 print("Starting global machine {0}".format(klass.global_machine.label))
             klass.global_machine.start()
@@ -719,62 +767,37 @@ class MachineCase(unittest.TestCase):
         return browser
 
     def checkSuccess(self):
-        if self._outcome:
-            # errors is a list of (method, exception) calls (usually multiple
-            # per method); None exception means success
-            return not any(e[1] for e in self._outcome.errors)
+        # errors is a list of (method, exception) calls (usually multiple
+        # per method); None exception means success
+        return not any(e[1] for e in self._outcome.errors)
 
-        if not self.currentResult:
-            return False
-        for error in self.currentResult.errors:
-            if self == error[0]:
-                return False
-        for failure in self.currentResult.failures:
-            if self == failure[0]:
-                return False
-        for success in self.currentResult.unexpectedSuccesses:
-            if self == success:
-                return False
-        for skipped in self.currentResult.skipped:
-            if self == skipped[0]:
-                return False
-        return True
+    def is_nondestructive(self):
+        test_method = getattr(self.__class__, self._testMethodName)
+        return getattr(test_method, "_testlib__non_destructive", False)
 
-    def run(self, result=None):
-        self.currentResult = result
+    def disable_preload(self, *packages):
+        for pkg in packages:
+            self.write_file("/usr/share/cockpit/%s/override.json" % pkg, '{ "preload": [ ] }')
 
-        # Here's the loop to actually retry running the test. It's an awkward
-        # place for this loop, since it only applies to MachineCase based
-        # TestCases. However for the time being there is no better place for it.
-        #
-        # Policy actually dictates retries.  The number here is an upper bound to
-        # prevent endless retries if Policy.check_retry is buggy.
-        max_retry_hard_limit = 10
-        for retry in range(0, max_retry_hard_limit):
-            try:
-                super().run(result)
-            except RetryError as ex:
-                assert retry < max_retry_hard_limit
-                sys.stderr.write("{0}\n".format(ex))
-                sleep(retry * 10)
-            else:
-                break
+    def enable_preload(self, package, *pages):
+        path = "/usr/share/cockpit/%s/override.json" % package
+        self.write_file(path, '{ "preload": [%s]}' % ', '.join('"{0}"'.format(page) for page in pages))
 
-        self.currentResult = None
-
-    def setUp(self):
+    def setUp(self, restrict=True):
         if opts.address and self.provision is not None:
             raise unittest.SkipTest("Cannot provision multiple machines if a specific machine address is specified")
 
         self.browser = None
         self.machines = {}
         provision = self.provision or {'machine1': {}}
+        self.tmpdir = tempfile.mkdtemp()
+        # automatically cleaned up for @nondestructive tests, but you have to create it yourself
+        self.vm_tmpdir = "/var/lib/cockpittest"
 
-        test_method = getattr(self.__class__, self._testMethodName)
-        if getattr(test_method, "_testlib__non_destructive", False) and not opts.address:
+        if self.is_nondestructive() and not opts.address:
             if self.provision:
                 raise unittest.SkipTest("Cannot provision machines if test is marked as nondestructive")
-            self.machine = self.machines['machine1'] = MachineCase.get_global_machine()
+            self.machine = self.machines['machine1'] = MachineCase.get_global_machine(restrict=restrict)
         else:
             self.machine = None
             # First create all machines, wait for them later
@@ -786,6 +809,8 @@ class MachineCase(unittest.TestCase):
                     del options['dns']
                 if 'dhcp' in options:
                     del options['dhcp']
+                if 'restrict' not in options:
+                    options['restrict'] = restrict
                 machine = self.new_machine(**options)
                 self.machines[key] = machine
                 if not self.machine:
@@ -794,14 +819,9 @@ class MachineCase(unittest.TestCase):
                     print("Starting {0} {1}".format(key, machine.label))
                 machine.start()
 
-        if self.machine.image == "rhel-8-2-distropkg":
-            self.danger_btn_class = '.btn-danger'
-            self.primary_btn_class = '.btn-primary'
-            self.default_btn_class = '.btn-default'
-        else:
-            self.danger_btn_class = '.pf-m-danger'
-            self.primary_btn_class = '.pf-m-primary'
-            self.default_btn_class = '.pf-m-secondary'
+        self.danger_btn_class = '.pf-m-danger'
+        self.primary_btn_class = '.pf-m-primary'
+        self.default_btn_class = '.pf-m-secondary'
 
         # Now wait for the other machines to be up
         for key in self.machines.keys():
@@ -818,42 +838,39 @@ class MachineCase(unittest.TestCase):
                 machine.dhcp_server()
 
         if self.machine:
+            # Pages with debug enabled are huge and loading/executing them is heavy for browsers
+            # To make it easier for browsers and thus make tests quicker, disable packagekit and systemd preloads
+            # Only "TEST_OS_DEFAULT" has debug build enabled, see `build_and_install()` in `test/image-prepare`
+            if self.machine.image == testvm.TEST_OS_DEFAULT:
+                self.disable_preload("packagekit", "systemd")
+
             self.journal_start = self.machine.journal_cursor()
             self.browser = self.new_browser()
-            if getattr(test_method, "_testlib__non_destructive", False):
+            # fail tests on criticals
+            self.machine.write("/etc/cockpit/cockpit.conf", "[Log]\nFatal = criticals\n")
+            if self.is_nondestructive():
                 self.nonDestructiveSetup()
 
-        self.tmpdir = tempfile.mkdtemp()
-
-        def sitter():
-            if opts.sit and not self.checkSuccess():
-                if self._outcome:
-                    [traceback.print_exception(*e[1]) for e in self._outcome.errors if e[1]]
-                else:
-                    self.currentResult.printErrors()
-                sit(self.machines)
-        self.addCleanup(sitter)
-
-        def intercept():
-            if not self.checkSuccess():
-                self.snapshot("FAIL")
-                self.copy_js_log("FAIL")
-                self.copy_journal("FAIL")
-                self.copy_cores("FAIL")
-        self.addCleanup(intercept)
-
     def nonDestructiveSetup(self):
         '''generic setUp/tearDown for @nondestructive tests'''
 
         m = self.machine
 
+        # helps with mapping journal output to particular tests
+        name = "%s.%s" % (self.__class__.__name__, self._testMethodName)
+        m.execute("logger -p user.info 'COCKPITTEST: start %s'" % name)
+        self.addCleanup(m.execute, "logger -p user.info 'COCKPITTEST: end %s'" % name)
+
+        # core dumps get copied per-test, don't clobber subsequent tests with them
+        self.addCleanup(m.execute, "rm -rf /var/lib/systemd/coredump")
+
+        # temporary directory in the VM
+        self.addCleanup(m.execute, "if [ -d {0} ]; then findmnt --list --noheadings --output TARGET | grep ^{0} | xargs -r umount && rm -r {0}; fi".format(self.vm_tmpdir))
+
         # users/groups/home dirs
-        m.execute("cp -a /etc/passwd /etc/passwd.test && "
-                  "cp -a /etc/shadow /etc/shadow.test && "
-                  "cp -a /etc/group /etc/group.test")
-        self.addCleanup(m.execute, "mv /etc/passwd.test /etc/passwd && "
-                                   "mv /etc/shadow.test /etc/shadow && "
-                                   "mv /etc/group.test /etc/group")
+        self.restore_file("/etc/passwd")
+        self.restore_file("/etc/group")
+        self.restore_file("/etc/shadow")
         home_dirs = m.execute("ls /home").strip().split()
 
         def cleanup_home_dirs():
@@ -864,16 +881,20 @@ class MachineCase(unittest.TestCase):
 
         # cockpit configuration
         self.addCleanup(m.execute, "rm -f /etc/cockpit/cockpit.conf")
+        self.addCleanup(m.execute, "rm -f /etc/cockpit/machines.d/*")
 
         if not m.ostree_image:
             # for storage tests
-            m.execute("cp -a /etc/fstab /etc/fstab.test && "
-                      "cp -a /etc/crypttab /etc/crypttab.test")
-            self.addCleanup(m.execute, "mv /etc/fstab.test /etc/fstab && "
-                                       "mv /etc/crypttab.test /etc/crypttab")
+            self.restore_file("/etc/fstab")
+            self.restore_file("/etc/crypttab")
 
             # tests expect cockpit.service to not run at start; also, avoid log leakage into the next test
-            self.addCleanup(m.execute, "systemctl stop cockpit")
+            self.addCleanup(m.execute, "systemctl stop --quiet cockpit")
+
+        # The sssd daemon seems to get confused when we restore
+        # backups of /etc/group etc and stops following updates to it.
+        # Let's restart the daemon to reset that condition.
+        m.execute("systemctl try-restart sssd || true")
 
         # reset scsi_debug (see e. g. StorageHelpers.add_ram_disk()
         # this needs to happen very late in the cleanup, so that test cases can clean up the users of that disk first
@@ -881,18 +902,27 @@ class MachineCase(unittest.TestCase):
         self.addCleanup(self.machine.execute,
                         "set -e; [ -e /sys/module/scsi_debug ] || exit 0; "
                         "for dev in $(ls /sys/bus/pseudo/drivers/scsi_debug/adapter*/host*/target*/*:*/block); do "
-                        "   umount /dev/$dev 2>/dev/null || true; "
+                        "    for s in /sys/block/*/slaves/${dev}*; do [ -e $s ] || break; "
+                        "        d=/dev/$(dirname $(dirname ${s#/sys/block/})); "
+                        "        umount $d || true; dmsetup remove --force $d || true; "
+                        "    done; "
+                        "    umount /dev/$dev 2>/dev/null || true; "
                         "done; until rmmod scsi_debug; do sleep 1; done")
 
+        # Terminate all lingering Cockpit sessions
+        self.addCleanup(self.machine.execute,
+                        "loginctl --no-legend list-sessions | awk '/web console/ { print $1 }' | "
+                        "xargs --no-run-if-empty -L1 loginctl terminate-session")
+
     def tearDown(self):
         if self.checkSuccess() and self.machine.ssh_reachable:
             self.check_journal_messages()
             self.check_browser_errors()
         shutil.rmtree(self.tmpdir)
 
-    def login_and_go(self, path=None, user=None, host=None, authorized=True, urlroot=None, tls=False):
+    def login_and_go(self, path=None, user=None, host=None, authorized=True, superuser=True, urlroot=None, tls=False):
         self.machine.start_cockpit(host, tls=tls)
-        self.browser.login_and_go(path, user=user, host=host, authorized=authorized, urlroot=urlroot, tls=tls)
+        self.browser.login_and_go(path, user=user, host=host, authorized=authorized, superuser=superuser, urlroot=urlroot, tls=tls)
 
     allow_core_dumps = False
 
@@ -940,9 +970,6 @@ class MachineCase(unittest.TestCase):
         "(audit: )?type=1400 .*denied.*comm=\"ssh-transport-c\".*name=\"unix\".*dev=\"proc\".*",
         "(audit: )?type=1400 .*denied.*comm=\"cockpit-ssh\".*name=\"unix\".*dev=\"proc\".*",
 
-        # https://bugzilla.redhat.com/show_bug.cgi?id=1795524
-        "(audit: )?type=1400 .*denied  { setsched } .* comm=\"cockpit-ws\".*",
-
         # apparmor loading
         "(audit: )?type=1400.*apparmor=\"STATUS\".*",
 
@@ -960,8 +987,20 @@ class MachineCase(unittest.TestCase):
 
         # our core dump retrieval is not entirely reliable
         "Failed to send coredump datagram:.*",
+
+        # Something crashed, but we don't have more info. Don't fail on that
+        "Failed to get (COMM|EXE).*: No such process",
+
+        # The usual sudo finger wagging
+        "We trust you have received the usual lecture from the local System",
+        "Administrator. It usually boils down to these three things:",
+        "#1\) Respect the privacy of others.",
+        "#2\) Think before you type.",
+        "#3\) With great power comes great responsibility.",
     ]
 
+    allowed_messages += os.environ.get("TEST_ALLOW_JOURNAL_MESSAGES", "").split(",")
+
     # Whitelist of allowed console.error() messages during tests; these match substrings
     allowed_console_errors = [
         # HACK: These should be fixed, but debugging these is not trivial, and the impact is very low
@@ -969,6 +1008,8 @@ class MachineCase(unittest.TestCase):
         "Warning: Can't perform a React state update on an unmounted component."
     ]
 
+    allowed_console_errors += os.environ.get("TEST_ALLOW_BROWSER_ERRORS", "").split(",")
+
     def allow_journal_messages(self, *patterns):
         """Don't fail if the journal contains a entry completely matching the given regexp"""
         for p in patterns:
@@ -1020,11 +1061,6 @@ class MachineCase(unittest.TestCase):
                                     "sudo: unable to stat /var/db/sudo: Permission denied",
                                     ".*: sorry, you must have a tty to run sudo",
                                     ".*/pkexec: bridge exited",
-                                    "We trust you have received the usual lecture from the local System",
-                                    "Administrator. It usually boils down to these three things:",
-                                    "#1\) Respect the privacy of others.",
-                                    "#2\) Think before you type.",
-                                    "#3\) With great power comes great responsibility.",
                                     ".*Sorry, try again.",
                                     ".*incorrect password attempt.*")
 
@@ -1040,13 +1076,17 @@ class MachineCase(unittest.TestCase):
         if "TEST_AUDIT_NO_SELINUX" not in os.environ:
             messages += machine.audit_messages("14", cursor=cursor) # 14xx is selinux
 
-        if self.image in ['fedora-32', 'fedora-31', 'fedora-30', 'fedora-testing']:
-            # Fedora >= 30 switched to dbus-broker
+        if self.image.startswith('debian'):
+            # Debian images don't have any non-C locales (mostly deliberate, to test this scenario somewhere)
+            self.allowed_messages.append("invalid or unusable locale: .*")
+
+        if self.image.startswith('fedora'):
+            # Fedora switched to dbus-broker
             self.allowed_messages.append("dbus-daemon didn't send us a dbus address; not installed?.*")
 
-        if self.image in ['rhel-8-2', 'rhel-8-2-distropkg']:
-            # HACK: https://bugzilla.redhat.com/show_bug.cgi?id=1753991
-            self.allowed_messages.append('.*type=1400.*avc:  denied  { dac_override } .* comm="rhsmd" .* scontext=system_u:system_r:rhsmcertd_t:s0-s0:c0.c1023 tcontext=system_u:system_r:rhsmcertd_t:.*')
+        if self.image in ['rhel-8-3', 'rhel-8-3-distropkg']:
+            # HACK: https://bugzilla.redhat.com/show_bug.cgi?id=1835909
+            self.allowed_messages.append('.*type=1400.*avc:  denied  { read write } .* comm="qemu-kvm" path="/dev/mapper/control".*')
 
         if self.image in ['debian-testing']:
             # HACK: https://bugs.debian.org/951477
@@ -1066,6 +1106,14 @@ class MachineCase(unittest.TestCase):
             if not m:
                 continue
             found = False
+
+            # When coredump could not be generated, we cannot do much with info about there being a coredump
+            # Ignore this message and all subsequent core dumps
+            # If there is more than just one line about coredump, it will fail and show this messages
+            if m == "Failed to generate stack trace: (null)":
+                self.allowed_messages.append("Process .* of user .* dumped core.*")
+                continue
+
             for p in self.allowed_messages:
                 match = re.match(p, m)
                 if match and match.group(0) == m:
@@ -1098,7 +1146,7 @@ class MachineCase(unittest.TestCase):
                 if re.search(p, log):
                     break
             else:
-                raise Error(log)
+                raise Error("FAIL: Test completed, but found unexpected browser errors:\n" + log)
 
     def check_axe(self, label=None, suffix=""):
         """Run aXe check on the currently active frame
@@ -1164,26 +1212,41 @@ class MachineCase(unittest.TestCase):
             title: Used for the filename.
         """
         if self.browser is not None:
-            self.browser.snapshot(title, label)
+            try:
+                self.browser.snapshot(title, label)
+            except RuntimeError:
+                # this usually runs in exception handlers; raising an exception here skips cleanup handlers, so don't
+                sys.stderr.write("Unexpected exception in snapshot():\n")
+                sys.stderr.write(traceback.format_exc())
 
     def copy_js_log(self, title, label=None):
         if self.browser is not None:
-            self.browser.copy_js_log(title, label)
+            try:
+                self.browser.copy_js_log(title, label)
+            except RuntimeError:
+                # this usually runs in exception handlers; raising an exception here skips cleanup handlers, so don't
+                sys.stderr.write("Unexpected exception in copy_js_log():\n")
+                sys.stderr.write(traceback.format_exc())
 
     def copy_journal(self, title, label=None):
         for name, m in self.machines.items():
             if m.ssh_reachable:
-                log = "%s-%s-%s.log" % (label or self.label(), m.label, title)
+                log = "%s-%s-%s.log.gz" % (label or self.label(), m.label, title)
                 with open(log, "w") as fp:
-                    m.execute("journalctl", stdout=fp)
+                    m.execute("journalctl|gzip", stdout=fp)
                     print("Journal extracted to %s" % (log))
                     attach(log)
 
     def copy_cores(self, title, label=None):
+        if self.allow_core_dumps:
+            return
         for name, m in self.machines.items():
             if m.ssh_reachable:
                 directory = "%s-%s-%s.core" % (label or self.label(), m.label, title)
                 dest = os.path.abspath(directory)
+                # overwrite core dumps from previous retries
+                if os.path.exists(dest):
+                    shutil.rmtree(dest)
                 m.download_dir("/var/lib/systemd/coredump", dest)
                 try:
                     os.rmdir(dest)
@@ -1206,6 +1269,88 @@ class MachineCase(unittest.TestCase):
                 break
             time.sleep(3)
 
+    def sed_file(self, expr, path, apply_change_action=None):
+        '''sed a file on primary machine
+
+        This is safe for @nondestructive tests, the file will be restored during cleanup.
+
+        The optional apply_change_action will be run both after sedding and after restoring the file.
+        '''
+        m = self.machine
+        m.execute("sed -i.cockpittest '{0}' {1}".format(expr, path))
+        if apply_change_action:
+            m.execute(apply_change_action)
+
+        if self.is_nondestructive():
+            if apply_change_action:
+                self.addCleanup(m.execute, apply_change_action)
+            self.addCleanup(m.execute, "mv {0}.cockpittest {0}".format(path))
+
+    def restore_dir(self, path, post_restore_action=None, reboot_safe=False):
+        '''Backup/restore a directory for a nondestructive test
+
+        This takes care to not ever touch the original content on disk, but uses transient overlays.
+        As this uses a bind mount, it does not work for files that get changed atomically (with mv);
+        use restore_file() for these.
+
+        The optional post_restore_action will run after restoring the original content.
+
+        If the directory needs to survive reboot, `reboot_safe=True` needs to be specified; then this
+        will just backup/restore the directory instead of bind-mounting, which is less robust.
+        '''
+        if not self.is_nondestructive():
+            return  # skip for efficiency reasons
+
+        exists = self.machine.execute("if test -e %s; then echo yes; fi" % path).strip() != ""
+        if not exists:
+            self.addCleanup(self.machine.execute, "rm -rf {0}".format(path))
+            return
+
+        backup = os.path.join(self.vm_tmpdir, path.replace('/', '_'))
+        self.machine.execute("mkdir -p %(vm_tmpdir)s && cp -a %(path)s/ %(backup)s/" % {
+            "vm_tmpdir": self.vm_tmpdir, "path": path, "backup": backup})
+
+        if not reboot_safe:
+            self.machine.execute("mount -o bind %(backup)s %(path)s" % {
+                "path": path, "backup": backup})
+
+        if post_restore_action:
+            self.addCleanup(self.machine.execute, post_restore_action)
+
+        if reboot_safe:
+            self.addCleanup(self.machine.execute, "rm -rf {0} && mv {1} {0}".format(path, backup))
+        else:
+            self.addCleanup(self.machine.execute, "umount -lf " + path)
+
+    def restore_file(self, path, post_restore_action=None):
+        '''Backup/restore a file for a nondestructive test
+
+        This is less robust than restore_dir(), but works for files that need to get changed atomically.
+
+        If path does not currently exist, it will be removed again on cleanup.
+        '''
+        exists = self.machine.execute("if test -e %s; then echo yes; fi" % path).strip() != ""
+        if exists:
+            backup = os.path.join(self.vm_tmpdir, path.replace('/', '_'))
+            self.machine.execute("mkdir -p %(vm_tmpdir)s && cp -a %(path)s %(backup)s" % {
+                "vm_tmpdir": self.vm_tmpdir, "path": path, "backup": backup})
+            if post_restore_action:
+                self.addCleanup(self.machine.execute, post_restore_action)
+            self.addCleanup(self.machine.execute, "mv %(backup)s %(path)s" % {"path": path, "backup": backup})
+        else:
+            self.addCleanup(self.machine.execute, "rm -rf %s" % path)
+
+    def write_file(self, path, content):
+        '''Write a new file on primary machine
+
+        This is safe for @nondestructive tests, the file will be removed during cleanup.
+        '''
+        m = self.machine
+        m.write(path, content)
+
+        if self.is_nondestructive():
+            self.addCleanup(m.execute, "rm -f {0}".format(path))
+
 
 def jsquote(str):
     return json.dumps(str)
@@ -1224,12 +1369,6 @@ def skipImage(reason, *args):
     return lambda func: func
 
 
-def allowImage(reason, *args):
-    if testvm.DEFAULT_IMAGE not in args:
-        return unittest.skip("{0}: {1}".format(testvm.DEFAULT_IMAGE, reason))
-    return lambda func: func
-
-
 def skipPackage(*args):
     packages_env = os.environ.get("TEST_SKIP_PACKAGES", "").split()
     for package in args:
@@ -1303,7 +1442,7 @@ class TapRunner:
         except Exception:
             result.addError(test, sys.exc_info())
             sys.stderr.write("Unexpected exception while running {0}\n".format(test))
-            sys.stderr.write(traceback.print_exc())
+            sys.stderr.write(traceback.format_exc())
             return result
         else:
             result.printErrors()
@@ -1373,7 +1512,7 @@ def print_tests(tests):
             print(test.id().replace("__main__.", ""))
 
 
-def arg_parser():
+def arg_parser(enable_sit=True):
     parser = argparse.ArgumentParser(description='Run Cockpit test(s)')
     parser.add_argument('-v', '--verbose', dest="verbosity", action='store_const',
                         const=2, help='Verbose output')
@@ -1381,12 +1520,15 @@ def arg_parser():
                         help='Trace machine boot and commands')
     parser.add_argument('-q', '--quiet', dest='verbosity', action='store_const',
                         const=0, help='Quiet output')
-    parser.add_argument('-s', "--sit", dest='sit', action='store_true',
-                        help="Sit and wait after test failure")
+    if enable_sit:
+        parser.add_argument('-s', "--sit", dest='sit', action='store_true',
+                            help="Sit and wait after test failure")
     parser.add_argument('--nonet', dest="fetch", action="store_false",
                         help="Don't go online to download images or data")
-    parser.add_argument('tests', nargs='*')
+    parser.add_argument('--enable-network', dest='enable_network', action='store_true',
+                        help="Enable network access for tests")
     parser.add_argument("-l", "--list", action="store_true", help="Print the list of tests that would be executed")
+    parser.add_argument('tests', nargs='*')
 
     parser.set_defaults(verbosity=1, fetch=True)
     return parser
@@ -1460,10 +1602,6 @@ class Error(Exception):
         return self.msg
 
 
-class RetryError(Error):
-    pass
-
-
 def wait(func, msg=None, delay=1, tries=60):
     """
     Wait for FUNC to return something truthy, and return that.
